(function () {

	'use strict';

	// sorting for options
	var $options_table = jQuery('.mpa-ctrl.mpa-options-ctrl');

	$options_table.sortable({
		items: 'tbody tr',
		cursor: 'move',
		handle: '.column-value, .column-label, .column-actions',
		axis: 'y',
		opacity: 0.65,
		scrollSensitivity: 40,
	});


	let globals = {};

	/**
	 * @param {String} name
	 * @param {String} className Optional. Additional class to set. '' by default.
	 * @return {String}
	 *
	 * @since 1.0
	 */
	function mpa_cf_tmpl_dashicon(name, className = '') {
		let classes = `dashicons dashicons-${name} ${className}`;
		return '<span class="' + classes.trimRight() + '"></span>';
	}

	/**
	 * @param {String} prefix Optional. '' by default.
	 * @param {Boolean} moreEntropy Optional. False by default.
	 * @return {String}
	 *
	 * @see https://locutus.io/php/misc/uniqid/
	 *
	 * @since 1.0
	 */

	function mpa_cf_uniqid(prefix = '', moreEntropy = false) {
		let mpa_cf_format_seed = function (seed, requiredWidth) {
			seed = parseInt(seed, 10).toString(16); // To hex string

			if (requiredWidth < seed.length) {
				// So long we split
				return seed.slice(seed.length - requiredWidth);
			} else if (requiredWidth > seed.length) {
				// So short we pad
				return Array(1 + (requiredWidth - seed.length)).join('0') + seed;
			} else {
				// Exact width
				return seed;
			}
		};
		if (!globals.uniqid_seed) {
			// Init seed with big random int
			globals.uniqid_seed = Math.floor(Math.random() * 0x75bcd15);
		}
		globals.uniqid_seed++;

		// Start with prefix
		let id = prefix;
		// Add current milliseconds (hex string)
		id += mpa_cf_format_seed(parseInt(new Date().getTime() / 1000, 10), 8);
		// Add seed hex string
		id += mpa_cf_format_seed(globals.uniqid_seed, 5);
		if (moreEntropy) {
			// For more entropy we add a float lowe to 10
			id += (Math.random() * 10).toFixed(8).toString();
		}
		return id;
	}

	class BasicField {
		/**
		 * @param {Object} $element
		 *
		 * @since 1.0
		 */
		constructor($element) {
			this.$element = $element;
			this.type = $element.data('type');
			this.$element.attr('data-inited', 'true');
		}
	}
	class OptionsField extends BasicField {
		/**
		 * @param {Object} $element
		 *
		 * @since 1.0
		 */
		constructor($element) {
			super($element);
			this.setupProperties();
			this.addListeners();
		}

		/**
		 * @access protected
		 *
		 * @since 1.0
		 */
		setupProperties() {

			this.$table = this.$element.find('table');
			this.$rows = this.$table.children('tbody');
			this.$addButton = this.$element.find('.mpa-add-button');
			this.baseName = this.$element.attr('data-base-name');
			this.rows = {}; // {Row ID: Row jQuery element}
			this.rowsCount = 0;

			// Find rows
			this.$element.find('.mpa-options-field-row').each((i, element) => {
				let $row = jQuery(element);
				let id = $row.attr('data-id');
				this.rows[id] = $row;
				this.rowsCount++;
			});
		}

		/**
		 * @access protected
		 *
		 * @since 1.0
		 */
		addListeners() {
			let self = this;

			// Add button
			this.$addButton.on('click', () => {
				this.addRow();
			});

			// Remove buttons
			this.$element.find('.mpa-remove-button').on('click', function () {
				self.removeRowByElement(this);
			});
		}

		/**
		 * @since 1.0
		 */
		addRow() {
			let id = mpa_cf_uniqid();
			// Add new row
			let newRowHtml = this.renderRow(id);
			this.$rows.append(newRowHtml);

			// Save new row
			let $row = this.$rows.find('[data-id="' + id + '"]');
			this.rows[id] = $row;
			this.rowsCount++;

			// Show table
			this.$table.removeClass('mpa-hide');

			// Add listeners for new buttons
			let self = this;
			$row.find('.mpa-remove-button').on('click', function () {
				self.removeRowByElement(this);
			});
		}

		/**
		 * @param {String} id New row ID.
		 * @return {String} New row HTML.
		 *
		 * @access protected
		 *
		 * @since 1.0
		 */
		renderRow(id) {
			let namePrefix = this.baseName + '[' + id + ']';
			let output = '';
			output += '<tr class="mpa-options-field-row" data-id="' + id + '">';
			output += '<td class="column-value">';
			output += '<input type="text" name="' + namePrefix + '[value]" value="" class="large-text">';
			output += '</td>';
			output += '<td class="column-label">';
			output += '<input type="text" name="' + namePrefix + '[label]" value="" class="large-text">';
			output += '</td>';
			output += '<td class="column-actions">' + mpa_cf_tmpl_dashicon('trash', 'mpa-remove-button') + '</td>';
			output += '</tr>';
			return output;
		}

		/**
		 * @param {Element} element
		 *
		 * @access protected
		 *
		 * @since 1.0
		 */
		removeRowByElement(element) {
			let rowId = jQuery(element).parents('.mpa-options-field-row').attr('data-id');
			if (rowId) {
				this.removeRow(rowId);
			}
		}

		/**
		 * @param {String} id Row ID.
		 *
		 * @since 1.0
		 */
		removeRow(id) {
			if (!this.rows.hasOwnProperty(id)) {
				return;
			}
			this.rows[id].remove();
			this.rowsCount--;
			delete this.rows[id];
			if (this.rowsCount == 0) {
				this.$table.addClass('mpa-hide');
			}
		}
	}

	class CheckoutFieldsEditPostPage {
		/**
		 * @since 1.0
		 */
		constructor() {
			this.setupFields(jQuery('.mpa-ctrl:not([data-inited])'));
		}

		/**
		 * @param {Object[]} $fields
		 *
		 * @access protected
		 *
		 * @since 1.0
		 */
		setupFields($fields) {

			$fields.each(function (i, element) {
				let $element = jQuery(element);
				let type = $element.attr('data-type');
				switch (type) {
					case 'options':
						new OptionsField($element);
						break;
				}
			});
		}
	}

	new CheckoutFieldsEditPostPage();
	
})();