<?php

namespace MotoPress\Appointment\CheckoutFields;

use MotoPress\Appointment\CheckoutFields\Plugin;
use MotoPress\Appointment\CheckoutFields\Admin\EddLicenseApi;
use MotoPress\Appointment\CheckoutFields\Fields\OptionsField;
use MotoPress\Appointment\CheckoutFields\CheckoutFieldsHelper;
use MotoPress\Appointment\CheckoutFields\Fields\FileUploadField;
use MotoPress\Appointment\CheckoutFields\Fields\DateOfBirthField;
use MotoPress\Appointment\CheckoutFields\Fields\LinkUploadButtonField;
use MotoPress\Appointment\CheckoutFields\Entities\CheckoutFieldBooking;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Adds actions and filters hooks of the plugin.
 */
class CheckoutFieldsHandler {

	const SETTINGS_TAB_SECTION_SLUG = 'mpa_checkout_fields_settings';

	public function __construct() {

		// add options fields
		add_filter( 'mpa_options_field_class', array( $this, 'options_field_class' ) );

		// add birth-date fields
		add_filter( 'mpa_date-of-birth_field_class', array( $this, 'date_of_birth_field_class' ) );

		// add upload-file fields
		add_filter( 'mpa_link-upload-button_field_class', array( $this, 'link_upload_button_field_class' ) );

		// Add checkout fields tab to appointment integration setings
		add_filter(
			'mpa_integrations_settings_tab_sections',
			function ( array $integrationsTabSections ) {

				$integrationsTabSections[ static::SETTINGS_TAB_SECTION_SLUG ] = __( 'Checkout Fields', 'mpa-checkout-fields' );

				return $integrationsTabSections;
			}
		);

		// Add fields to appointment integration setings tab
		add_filter(
			'mpa_integrations_section_settings',
			function ( array $settingsFields, string $currentSettingsTabSection ) {

				if ( static::SETTINGS_TAB_SECTION_SLUG === $currentSettingsTabSection ) {

					$settingsFields += $this->getSettingsFields();
				}

				return $settingsFields;
			},
			10,
			2
		);

		add_filter(
			'mpa_cf_customer_fields',
			function ( array $customerFields ): array {
				return $this->filterCustomerFields( $customerFields );
			},
			10,
			3
		);

		// Checkout Fields List Page
		add_filter(
			'edit_mpa_checkout_field_per_page',
			function ( $checkoutFieldsPerPage ) {
				return 999; // show all fields on a single page
			},
			10,
			1
		);

		// Edit actions of Checkout Fields List row
		add_filter(
			'post_row_actions',
			function ( array $actions, \WP_Post $wpPost ): array {
				return $this->filterCheckoutFieldsListRowActions( $actions, $wpPost );
			},
			10,
			2
		);

		add_filter(
			'pre_trash_post',
			function ( $check, \WP_Post $wpPost ) {
				return $this->preventRemovalOfDefaultCheckoutFields( $check, $wpPost );
			},
			10,
			2
		);

		add_filter(
			'pre_delete_post',
			function ( $check, \WP_Post $wpPost ) {
				return $this->preventRemovalOfDefaultCheckoutFields( $check, $wpPost );
			},
			10, // if you change this priority then make sure you do not forbid uninstall fields deletion in PluginLifecycleHandler
			2
		);

		// Enqueue script on Edit Booking page
		add_action(
			'admin_enqueue_scripts',
			function () {
				if ( mpapp()->pages()->getEditPage( 'mpa_booking' )->isCurrentEditPage() ) {
					wp_enqueue_script(
						'mpa-cf-date-of-birth',
						CheckoutFieldsHelper::getUrlToFile( 'assets/js/date-of-birth-control.js' ),
						array( 'jquery' ),
						Plugin::getInstance()->getPluginVersion(),
						true
					);
				}
			}
		);

		add_action(
			'before_delete_post',
			function ( int $wpPostId ) {
				FileUploadField::deleteUploadedCheckoutFieldsFilesOfBooking( $wpPostId );
			},
			10,
			1
		);

		// Add checkout fields to save booking
		add_filter( 'mpa_pre_save_booking', array( $this, 'add_cf_to_save_booking' ), 10, 2 );

		// Add checkout fields to save booking drafts
		add_filter( 'mpa_pre_save_booking_drafts', array( $this, 'add_cf_to_save_booking' ), 10, 2 );

		// Save checkout fields data to booking meta
		add_action( 'mpa_update_booking_post_meta', array( $this, 'add_cf_to_booking_meta' ) );

		// file uploads
		add_action( 'wp_ajax_mpa_cf_uploadFile', array( FileUploadField::class, 'uploadFile' ) );
		add_action( 'wp_ajax_nopriv_mpa_cf_uploadFile', array( FileUploadField::class, 'uploadFile' ) );

		add_action( 'init', array( FileUploadField::class, 'processViewUploadedFileRequest' ) );

		// admin booking file upload
		add_action( 'save_post', array( FileUploadField::class, 'saveAdminBookingCheckoutFieldsFileUploads' ) );

		add_action( 'admin_head-post.php', array( $this, 'add_enctype_to_mpa_booking_admin_form' ) );
		add_action( 'admin_head-post-new.php', array( $this, 'add_enctype_to_mpa_booking_admin_form' ) );

		// Export csv
		add_filter( 'mpa_get_booking_export_columns', array( $this, 'add_mpa_cf_export_csv_columns' ) );
		add_filter( 'mpa_get_booking_reservation_export_data', array( $this, 'add_mpa_cf_export_csv_data' ), 10, 2 );

		// Add notices for admin file uploads
		add_action(
			'admin_notices',
			function () {
				if ( $errors = get_transient( 'mpa_checkout_fields_admin_upload_errors' ) ) {
					foreach ( $errors as $error_message ) {
						echo '<div class="notice notice-warning is-dismissible">';
						echo '<p>' . esc_html( $error_message ) . '</p>';
						echo '</div>';
					}
					delete_transient( 'mpa_checkout_fields_admin_upload_errors' );
				}
			}
		);

		// Enqueue script for admin file upload
		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_admin_upload_script' ) );
	}

	/**
	 * Enqueue script for admin file upload
	 */
	public function enqueue_admin_upload_script() {
		global $post_type;
		if ( 'mpa_booking' == $post_type ) {
			wp_enqueue_script(
				'admin-file-upload',
				CheckoutFieldsHelper::getUrlToFile( 'assets/js/admin-file-upload.js' ),
				array( 'jquery' ),
				Plugin::getInstance()->getPluginVersion(),
				true
			);

			wp_localize_script(
				'admin-file-upload',
				'MPAUploadParams',
				array(
					'textNoFile'    => esc_html__( 'No file selected yet', 'mpa-checkout-fields' ),
					'textMaxUpload' => esc_html__( 'The file size exceeds the maximum upload size of ', 'mpa-checkout-fields' ),
					'maxUploadSize' => wp_max_upload_size(),
				)
			);
		}
	}

	/**
	 * Add checkout fields to csv export columns
	 */
	public function add_mpa_cf_export_csv_columns( $columns ) {

		$fields = mpa_checkout_fields_get_custom_customer_fields();

		// add checkout fields columns
		foreach ( $fields as $key => $value ) {

			$columns[ $key ] = ! empty( $value['label'] ) ? $value['label'] : $key;
		}

		return $columns;
	}

	/**
	 * Add checkout fields to csv export data
	 */
	public function add_mpa_cf_export_csv_data( $reservationExportData ) {

		$booking_id = $reservationExportData['booking_id'];

		$fields = mpa_checkout_fields_get_custom_customer_fields();

		foreach ( $fields as $key => $val ) {

			$value = get_post_meta( $booking_id, '_mpa_checkout_fields_' . $key, true );

			switch ( $val['type'] ) {

				case 'country':
					$value = mpapp()->bundles()->countries()->getLabel( $value );
					break;

				case 'date_of_birth':
					$date = \DateTime::createFromFormat( 'Y-m-d', $value );
					if ( $date !== false ) {
						$value = date_i18n( mpapp()->settings()->getDateFormat(), $date->getTimestamp() );
					}
					break;

				case 'checkbox':
					if ( $value === '1' ) {
						$value = esc_html__( 'yes', 'mpa-checkout-fields' );
					} elseif ( $value === '0' ) {
						$value = esc_html__( 'no', 'mpa-checkout-fields' );
					}
					break;

				case 'file_upload':
					if ( $value ) {
						$value = FileUploadField::getUploadedFileLink( $booking_id, $key );
					}
					break;

			}

			$reservationExportData[ $key ] = $value;
		}

		return $reservationExportData;
	}

	// admin booking file upload
	public function add_enctype_to_mpa_booking_admin_form() {
		global $post_type;
		if ( 'mpa_booking' == $post_type ) {
			?>
			<script type="text/javascript">
				jQuery(document).ready(function($) {
					$('#post').attr('enctype', 'multipart/form-data');
					$('#post').attr('encoding', 'multipart/form-data');
				});
			</script>
			<?php
		}
	}

	/**
	 * Add options field class
	 */
	public function options_field_class() {
		return OptionsField::class;
	}
	/**
	 * Add birth_date field class
	 */
	public function date_of_birth_field_class() {
		return DateOfBirthField::class;
	}
	/**
	 * Add link_button field class
	 */
	public function link_upload_button_field_class() {
		return LinkUploadButtonField::class;
	}

	// Settings fields to appointment integration setings tab
	private function getSettingsFields(): array {

		$settingsFields = array(
			'mpa_checkout_fields_settings_group' => array(
				'type'  => 'group',
				'label' => __( 'Checkout Fields', 'mpa-checkout-fields' ),
			),
		);

		if ( Plugin::getInstance()->isEDDLicenseEnabled() ) {
			$settingsFields[ EddLicenseApi::OPTION_KEY ] = array(
				'type'         => 'checkout-fields-license-settings',
				'label'        => __( 'License Key', 'mpa-checkout-fields' ),
				'description'  => __( "<i>The License Key is required in order to get automatic plugin updates and support. You can manage your License Key in your personal account. <a href='https://motopress.zendesk.com/hc/en-us/articles/202812996-How-to-use-your-personal-MotoPress-account' target='blank'>Learn more</a>.</i>", 'mpa-checkout-fields' ),
				'size'         => 'regular',
				'translatable' => false,

			);
		}

		return $settingsFields;
	}

	/**
	 * Adds custom checkout fields to the built-in Appointment Booking plugin customer fields
	 */
	private function filterCustomerFields( array $customerFields ): array {

		$allCustomFields = CheckoutFieldsHelper::getEnabledCheckoutFields();
		$checkoutFields  = array();

		foreach ( $allCustomFields as $name => $field ) {

			// We don't need abstract fields in the array for parsing and validation
			if ( in_array( $field->type, array( 'heading', 'paragraph' ) ) ) {
				continue;
			}

			if ( isset( $customerFields[ $name ] ) ) {
				// Copy labels and type, but update "enabled" and "required"
				$checkoutFields[ $name ] = $customerFields[ $name ];

				$checkoutFields[ $name ]['enabled']  = $field->isEnabled;
				$checkoutFields[ $name ]['required'] = $field->isRequired;

			} else {

				$checkoutFields[ $name ] = array(
					'label'       => $field->title,
					'type'        => $field->type,
					'enabled'     => $field->isEnabled,
					'required'    => $field->isRequired,
					'labels'      => array(
						// Translators: %s is a field name like "birth_date".
						'required_error' => sprintf( esc_html__( 'The field "%s" is required.', 'mpa-checkout-fields' ), $name ),
					),
					'file_types'  => $field->fileTypes,
					'upload_size' => $field->uploadSize,
				);
			}
		}

		return $checkoutFields;
	}

	/**
	 * Removes "Quick Edit" and maybe the "Trash" actions on the Checkout Fields List page
	 */
	private function filterCheckoutFieldsListRowActions( array $actions, \WP_Post $wpPost ): array {

		if ( ! CheckoutFieldsHelper::isCheckoutFieldPost( $wpPost ) ) {
			return $actions;
		}

		if ( isset( $actions['inline hide-if-no-js'] ) ) {
			unset( $actions['inline hide-if-no-js'] );
		}

		if ( isset( $actions['trash'] ) && CheckoutFieldsHelper::isDefaultCheckoutFieldPost( $wpPost ) ) {
			unset( $actions['trash'] );
		}

		return $actions;
	}

	/**
	 * @param mixed $check NULL by default
	 * @return mixed bool or NULL
	 */
	private function preventRemovalOfDefaultCheckoutFields( $check, \WP_Post $wpPost ) {

		return CheckoutFieldsHelper::isDefaultCheckoutFieldPost( $wpPost ) ? true : $check;
	}

	/**
	 * Add checkout fields to save booking
	 *
	 * @param mixed $booking The booking object.
	 * @param array $order The order array containing checkout fields.
	 *
	 * @return mixed The updated booking object.
	 */
	public function add_cf_to_save_booking( $booking, $order ) {
		if ( isset( $order['checkout_fields'] ) ) {
			$checkoutFieldsInOrder = $order['checkout_fields'];

			foreach ( $checkoutFieldsInOrder as $key => $value ) {
				$value                         = sanitize_text_field( wp_unslash( $value ) );
				$checkoutFieldsInOrder[ $key ] = $value;
			}

			if ( $booking instanceof CheckoutFieldBooking ) {
				$booking->setCheckoutFields( $checkoutFieldsInOrder );
			} else {
				$newBooking = CheckoutFieldBooking::fromBooking( $booking );
				$newBooking->setCheckoutFields( $checkoutFieldsInOrder );

				return $newBooking;
			}
		}

		return $booking;
	}

	/**
	 * Save checkout fields to booking meta
	 */
	public function add_cf_to_booking_meta( $booking ) {

		$postId = $booking->getId();

		if ( isset( $booking->checkout_fields ) ) {
			$checkout_fields = $booking->checkout_fields;

			foreach ( $checkout_fields as $key => $value ) {
				update_post_meta( $postId, '_mpa_checkout_fields_' . $key, $value );
			}
		}
	}
}
