<?php

namespace MotoPress\Appointment\CheckoutFields\Admin;

use MotoPress\Appointment\CheckoutFields\Plugin;
use MotoPress\Appointment\CheckoutFields\Admin\EddPluginUpdater;
use MotoPress\Appointment\CheckoutFields\Bundles\CustomerBundle;
use MotoPress\Appointment\CheckoutFields\Fields\FileUploadField;
use MotoPress\Appointment\CheckoutFields\UsersAndRoles\Capabilities;
use const MotoPress\Appointment\CheckoutFields\PLUGIN_DIR as MPA_CF_PLUGIN_DIR;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}


class PluginLifecycleHandler {

	private const REQUIRED_APPOINTMENT_PLUGIN_VERSION = '2.1.0';

	const OPTION_NAME_UPGRADED_PLUGIN_VERSION = 'mpacf_upgraded_version';

	private $isWPEnvironmentSuitedForPlugin = true;

	private $pluginMainFilePath = '';

	public function __construct( string $pluginMainFilePath ) {

		$this->pluginMainFilePath = $pluginMainFilePath;

		add_action(
			'plugins_loaded',
			function () {
				$this->checkIsWPEnvironmentSuitedForPlugin();
			},
			-1
		);

		register_activation_hook(
			$this->pluginMainFilePath,
			function ( $isNetworkWide = false ) {
				$this->activatePlugin( $isNetworkWide );
			}
		);

		add_action(
			'mpa_activated',
			function () {
				$this->activatePlugin();
			}
		);

		// add installation for a new site in multisite WordPress
		add_action(
			version_compare( get_bloginfo( 'version' ), '5.1', '>=' ) ? 'wp_initialize_site' : 'wpmu_new_blog',
			/**
			 * @param $blog in case of wp_initialize_site action is WP_Site otherwise int (site id)
			 */
			function ( $blog ) {
				$blogId = is_int( $blog ) ? $blog : $blog->blog_id;
				$this->activatePlugin( false, $blogId );
			}
		);

		register_deactivation_hook(
			$this->pluginMainFilePath,
			function () {
				$this->deactivatePlugin();
			}
		);

		register_uninstall_hook( $this->pluginMainFilePath, array( __CLASS__, 'uninstallPlugin' ) );

		// initialize EDD updater
		if ( ! wp_doing_ajax() && ! wp_doing_cron() ) {

			add_action(
				'admin_init',
				function () {

					if ( Plugin::getInstance()->isEDDLicenseEnabled() ) {

						new EddPluginUpdater(
							Plugin::getInstance()->getPluginSourceServerUrl(),
							$this->pluginMainFilePath,
							array(
								'version' => Plugin::getInstance()->getPluginVersion(),
								'license' => Plugin::getInstance()->getEddLicenseApi()->getKey(),
								'item_id' => Plugin::getInstance()->getEddLicenseApi()->getProductId(),
								'author'  => Plugin::getInstance()->getPluginAuthorName(),
							)
						);
					}
				}
			);
		}

		// check and upgrade plugin data if it is needed after plugin update
		add_action(
			'init',
			function () {
				if ( Plugin::getInstance()->settings()->isActivated() && $this->isWPEnvironmentSuitedForPlugin() ) {

					$this->upgradePluginData();
				}
			}
		);
	}


	private function checkIsWPEnvironmentSuitedForPlugin() {

		$isRequiredVersionOfAppointmentPluginActive = false;

		if ( defined( 'MotoPress\Appointment\VERSION' ) ) {

			$isRequiredVersionOfAppointmentPluginActive = version_compare(
				\MotoPress\Appointment\VERSION,
				static::REQUIRED_APPOINTMENT_PLUGIN_VERSION,
				'>='
			);
		}

		if ( ! $isRequiredVersionOfAppointmentPluginActive ) {

			$this->isWPEnvironmentSuitedForPlugin = false;

			global $pagenow;

			if ( isset( $pagenow ) && 'plugins.php' === $pagenow ) {

				$this->addErrorAdminNotice(
					sprintf(
						// translators: %1$s is the name of the required plugin, %2$s is the required plugin version like 1.x.x
						esc_html__( 'The Appointment Booking Checkout Fields addon requires the %1$s plugin version %2$s or higher. Install and activate it for proper work.', 'mpa-checkout-fields' ),
						'Appointment Booking',
						esc_html( static::REQUIRED_APPOINTMENT_PLUGIN_VERSION )
					)
				);
			}
		}

		return $this->isWPEnvironmentSuitedForPlugin;
	}

	private function addErrorAdminNotice( string $errorText ): void {

		add_action(
			'admin_notices',
			function () use ( $errorText ) {
				echo '<div class="notice notice-warning">
						<p>' . esc_html( $errorText ) . '</p>
					</div>';
			}
		);
	}

	/**
	 * @return bool
	 */
	public function isWPEnvironmentSuitedForPlugin() {

		return $this->isWPEnvironmentSuitedForPlugin;
	}

	private function activatePlugin( $isNetworkWide = false, $multisiteBlogId = 0 ) {

		// check environment because plugin was not loaded yet before activation
		if ( ! $this->checkIsWPEnvironmentSuitedForPlugin() ) {

			return;
		}

		if ( is_multisite() && $isNetworkWide ) {

			$sites = get_sites();

			foreach ( $sites as $site ) {

				switch_to_blog( $site->blog_id );

				$this->installPlugin();

				restore_current_blog();
			}
		} elseif ( is_multisite() && 0 < $multisiteBlogId ) {

			if ( ! is_plugin_active_for_network( plugin_basename( $this->pluginMainFilePath ) ) ) {
				return;
			}

				switch_to_blog( $multisiteBlogId );

				$this->installPlugin();

				restore_current_blog();

		} else {

			$this->installPlugin();
		}
	}

	public function installPlugin() {

		if ( Plugin::getInstance()->settings()->isActivated() ) {
			return;
		}

		// Get Appointment checkout default fields
		$CustomerBundle = new CustomerBundle();
		$defaultFields  = $CustomerBundle->getDefaultFields();
		$postType       = Plugin::getInstance()->getCheckoutFieldsPostType()->getPostType();

		// we try to find previously stored fields if it is not a first plugin installation
		// to avoide duplicated fields in the database because of old plugin versions
		$alreadyStoredCheckoutFields = Plugin::getInstance()->getCheckoutFieldRepository()->findAll();

		$order = 1;

		// Add Appointment checkout fields to the database
		foreach ( $defaultFields as $name => $args ) {

			$storingFieldData = array(
				'post_type'   => $postType,
				'post_title'  => $args['label'],
				'post_status' => 'publish',
				'menu_order'  => $order,
				'post_author' => 0,
				'meta_input'  => array(
					'mpa_cf_name'     => $name,
					'mpa_cf_type'     => $args['type'],
					'mpa_cf_enabled'  => (int) $args['enabled'],
					'mpa_cf_required' => (int) $args['required'],
				),
			);

			if ( ! empty( $alreadyStoredCheckoutFields ) ) {

				foreach ( $alreadyStoredCheckoutFields as $alreadyStoredField ) {

					if ( $name == $alreadyStoredField->name ) {

						$storingFieldData['ID'] = $alreadyStoredField->getId();
						break;
					}
				}
			}

			wp_insert_post( $storingFieldData );

			++$order;
		}

		$this->upgradePluginData();

		Plugin::getInstance()->settings()->setActivated();
	}

	private function upgradePluginData() {

		$upgradedPluginVersion = get_option( static::OPTION_NAME_UPGRADED_PLUGIN_VERSION );

		if ( $upgradedPluginVersion && version_compare( $upgradedPluginVersion, Plugin::getInstance()->getPluginVersion(), '>=' ) ) {

			return;
		}

		try {

			// Add plugin's custom roles and capabilities
			Capabilities::setup();

			$this->createUploadsFolder();

			update_option( static::OPTION_NAME_UPGRADED_PLUGIN_VERSION, Plugin::getInstance()->getPluginVersion() );

		} catch ( \Throwable $e ) {

			error_log( 'ERROR: ' . $e->getMessage() . PHP_EOL . $e->getTraceAsString() );
			$this->addErrorAdminNotice( $e->getMessage() );
		}
	}

	/**
	 * @throws Exception when uploads directory was not created
	 */
	private function createUploadsFolder() {

		$uploadFolder = wp_upload_dir()['basedir'] . '/' . FileUploadField::SECRET_UPLOADS;

		if ( file_exists( $uploadFolder ) ) {
			return;
		}

		require_once ABSPATH . 'wp-admin/includes/file.php';

		WP_Filesystem();

		if ( wp_mkdir_p( $uploadFolder ) ) {

			$result = copy_dir(
				MPA_CF_PLUGIN_DIR . 'assets/others/' . FileUploadField::SECRET_UPLOADS,
				$uploadFolder
			);

			if ( true !== $result ) {
				throw new \Exception( esc_html__( 'Could not copy .htaccess file to the created uploads directory.', 'mpa-checkout-fields' ) );
			}
		} else {

			throw new \Exception( esc_html__( 'Could not create uploads directory.', 'mpa-checkout-fields' ) );
		}
	}

	private function deactivatePlugin() {}

	public static function uninstallPlugin() {

		update_option( static::OPTION_NAME_UPGRADED_PLUGIN_VERSION, null );
		update_option( 'mpa_checkout_fields_activated', false, false );
	}
}