<?php

namespace MotoPress\Appointment\CheckoutFields\Fields;

use MotoPress\Appointment\Utils\ParseUtils;
use MotoPress\Appointment\Fields\AbstractField;

/**
 * @since 1.0
 */
class DateOfBirthField extends AbstractField {

	const TYPE = 'date-of-birth';

	/**
	 * @var int
	 */
	protected $minAge = 0;

	/**
	 * @var int
	 */
	protected $maxAge = 100;

	protected $default  = '';
	protected $value    = '';
	protected $htmlid   = '';
	protected $required = false;


	/**
	 * @param string $inputName Prefixed name.
	 * @param array  $args
	 * @param mixed  $value Optional. Null by default.
	 */
	public function __construct( $inputName, $args, $value = null ) {
		parent::__construct( $inputName, $args, $value );

		$this->htmlid   = isset( $args['htmlid'] ) ? $args['htmlid'] : '';
		$this->required = isset( $args['required'] ) ? $args['required'] : '';
		$this->minAge   = isset( $args['min_age'] ) ? $args['min_age'] : apply_filters( 'mpa_cf_date_of_birth_min_age', $this->minAge );
		$this->maxAge   = isset( $args['max_age'] ) ? $args['max_age'] : apply_filters( 'mpa_cf_date_of_birth_max_age', $this->maxAge );
	}

	/**
	 * @return string
	 */
	public function renderInput() {

		return $this->renderFields();
	}

	/**
	 * @return string
	 *
	 * @since 1.0.0
	 */
	public function renderFields() {

			$output  = $this->renderDays();
			$output .= '&nbsp;';
			$output .= $this->renderMonths();
			$output .= '&nbsp;';
			$output .= $this->renderYears();

		return $output;
	}

		/**
		 * @return string
		 */
	protected function renderDays() {
		$atts = $this->inputAtts();

		$atts['name']  = is_admin() ? '_mpa_' . $atts['name'] : $atts['name'];
		$atts['name'] .= '[day]';
		$atts['id']   .= is_admin() ? '-day' : '-day-' . $this->htmlid;
		$atts['class'] = 'mpa-control-component mpa-day-control-component';

		$values      = $this->getValues();
		$daysInMonth = mpa_checkout_fields_days_in_month( $values['month'], $values['year'] );
		$daysRange   = range( 1, $daysInMonth );

		$days  = array( '' => esc_html__( 'Day', 'mpa-checkout-fields' ) );
		$days += array_combine( $daysRange, $daysRange );

		$selectedDay = ! empty( $this->value ) ? $values['day'] : '';

		return mpa_tmpl_select( $days, $selectedDay, $atts );
	}

	/**
	 * @return string
	 */
	protected function renderMonths() {
		$atts = $this->inputAtts();

		$atts['name']  = is_admin() ? '_mpa_' . $atts['name'] : $atts['name'];
		$atts['name'] .= '[month]';
		$atts['id']   .= is_admin() ? '-month' : '-month-' . $this->htmlid;
		$atts['class'] = 'mpa-control-component mpa-month-control-component';

		$months = array( '' => esc_html__( 'Month', 'mpa-checkout-fields' ) );

		$gregorianCalendar = cal_info( CAL_GREGORIAN );
		$months           += array_map( 'translate', $gregorianCalendar['months'] );

		$selectedMonth = ! empty( $this->value ) ? $this->getMonth() : '';

		return mpa_tmpl_select( $months, $selectedMonth, $atts );
	}

	/**
	 * @return string
	 */
	protected function renderYears() {
		$atts          = $this->inputAtts();
		$atts['name']  = is_admin() ? '_mpa_' . $atts['name'] : $atts['name'];
		$atts['name'] .= '[year]';
		$atts['id']   .= is_admin() ? '-year' : '-year-' . $this->htmlid;
		$atts['class'] = 'mpa-control-component mpa-year-control-component';

		$currentYear = mpa_current_year();
		$yearsRange  = range( $currentYear - $this->minAge, $currentYear - $this->maxAge );

		$years  = array( '' => esc_html__( 'Year', 'mpa-checkout-fields' ) );
		$years += array_combine( $yearsRange, $yearsRange );

		$selectedYear = ! empty( $this->value ) ? $this->getYear() : '';

		return mpa_tmpl_select( $years, $selectedYear, $atts );
	}

	protected function inputAtts() {
		$atts = array(
			'name' => $this->name,
			'id'   => $this->name,
		);

		if ( $this->required ) {
			$atts['required'] = 'required';
		}

		return $atts;
	}

	/**
	 * @param string|array $value
	 * @return string
	 */
	public function validateValue( $value ) {
		// Remove empty values before the next check
		if ( is_array( $value ) ) {
			$value = array_filter( $value );
		}

		// Return "" for empty values
		if ( empty( $value ) ) {
			return '';
		}

		// Split the value into [day, month, year] values
		if ( is_string( $value ) ) {
			preg_match( '/(?<year>\d+)?\-?(?<month>\d+)?\-?(?<day>\d+)?/', $value, $date );
		} else {
			$date = $value;
		}

		// Validate
		$currentYear = mpa_current_year();

		$day   = 1;
		$month = 1;
		$year  = $currentYear - $this->minAge;

		foreach ( array( 'year', 'month', 'day' ) as $fieldName ) {
			$fieldValue = isset( $date[ $fieldName ] ) ? $date[ $fieldName ] : '';

			if ( $fieldValue !== '' ) {
				// filter_var() in ParseUtils::parseInt() fails on strings
				// with leading zeros when trying to validate an integer number
				$fieldValue = ltrim( $fieldValue, '0' );

				switch ( $fieldName ) {
					case 'year':
						$year = ParseUtils::parseInt( $fieldValue );
						$year = mpa_checkout_fields_limit( $year, $currentYear - $this->maxAge, $currentYear - $this->minAge );
						break;

					case 'month':
						$month = ParseUtils::parseInt( $fieldValue );
						$month = mpa_checkout_fields_limit( $month, 1, 12 );
						break;

					case 'day':
						$day         = ParseUtils::parseInt( $fieldValue );
						$daysInMonth = mpa_checkout_fields_days_in_month( $month, $year );
						$day         = mpa_checkout_fields_limit( $day, 1, $daysInMonth );
						break;
				}
			} // if $fieldValue !== ''
		} // For each field

		$day   = str_pad( $day, 2, '0', STR_PAD_LEFT );
		$month = str_pad( $month, 2, '0', STR_PAD_LEFT );

		return "{$year}-{$month}-{$day}";
	}

	/**
	 * @return int[] [year, month, day]
	 */
	public function getValues() {
		$values = ! empty( $this->value ) ? explode( '-', $this->value ) : array();

		return array(
			'year'  => isset( $values[0] ) ? (int) $values[0] : mpa_current_year() - $this->minAge,
			'month' => isset( $values[1] ) ? (int) $values[1] : 1,
			'day'   => isset( $values[2] ) ? (int) $values[2] : 1,
		);
	}

	/**
	 * @param mixed $value
	 * @param mixed $validate Optional. False by default.
	 */
	public function setValue( $value, $validate = false ) {

		parent::setValue( $value, $validate );
	}

	/**
	 * @return int
	 */
	public function getDay() {
		$values = $this->getValues();

		return $values['day'];
	}

	/**
	 * @return int
	 */
	public function getMonth() {
		$values = $this->getValues();

		return $values['month'];
	}

	/**
	 * @return int
	 */
	public function getYear() {
		$values = $this->getValues();

		return $values['year'];
	}
}
