<?php

namespace MotoPress\Appointment\CheckoutFields\Fields;

use MotoPress\Appointment\Fields\AbstractField;
use MotoPress\Appointment\CheckoutFields\CheckoutFieldsHelper;

/**
 *
 * @since 1.1.0
 */
class FileUploadField extends AbstractField {

	const TYPE = 'file-upload';

	const SECRET_UPLOADS = 'mpa_protected_uploads';

	/**
	 * @var string
	 */
	protected $value = '';

	private $fileTypes;
	private $uploadSize;
	private $html_id;
	private $required;

	public function __construct( $name, $args, $value = '' ) {

		parent::__construct( $name, $args, $value );

		$this->html_id    = isset( $args['html_id'] ) ? $args['html_id'] : '';
		$this->fileTypes  = isset( $args['file_types'] ) ? $args['file_types'] : array();
		$this->uploadSize = isset( $args['upload_size'] ) ? (int) $args['upload_size'] : 0;
		$this->required   = isset( $args['required'] ) ? $args['required'] : '';
	}

	/**
	 * @return string
	 */
	public function renderInput() {
		return $this->renderFields();
	}

	public function renderFields(): string {

		$inputAtts = array(
			'name'  => $this->name,
			'id'    => 'mpa-customer-' . $this->name . '-' . $this->html_id,
			'class' => 'mpa-customer-' . $this->name . ' resultField',
		);

		if ( $this->required ) {
			$inputAtts['required'] = 'required';
		}

		$output  = '<input type="file" ' . mpa_tmpl_atts( $inputAtts ) . ' autocomplete="off" />';
		$output .= $this->renderUploadFileSize();
		$output .= $this->renderUploadFileType();
		$output .= '<br><span class="mpa-upload-status"></span>';

		return $output;
	}

	protected function renderUploadFileSize() {

		$output = '<br><span class="mpa-max-upload-file">';

		$output .= sprintf(
			/* translators: %s: Maximum allowed file size. */
			sprintf( esc_html__( 'Maximum upload file size: %s.', 'mpa-checkout-fields' ), size_format( $this->uploadSize ) )
		);

		$output .= '</span>';

		return $output;
	}

	protected function renderUploadFileType() {

		$output = '';

		$types = $this->fileTypes;

		if ( ! empty( $types ) && is_array( $types ) ) {
			$fileTypesStr = implode( ', ', $types );

			$output .= '<br><span class="mpa-accepted-upload-types">';

			$output .= sprintf(
				/* translators: %s: Accepted file types. */
				sprintf( esc_html__( 'Accepted file types: %s.', 'mpa-checkout-fields' ), $fileTypesStr )
			);

			$output .= '</span>';
		}

		return $output;
	}

	public static function uploadFile() {

		// Check nonce for security
		check_ajax_referer( 'mpa_cf_file_upload_nonce', 'file_upload_nonce' );

		$MpappConnfirmationMode  = mpapp()->settings()->getConfirmationMode();
		$wpHandleUploadOverrides = array( 'test_form' => false );
		$fields                  = CheckoutFieldsHelper::getEnabledCheckoutFields();
		$response                = array();

		// Delete previously uploaded on drafts
		if ( isset( $_POST['booking_id'] ) && ! empty( $_POST['booking_id'] ) ) {
			$activeFieldNames = array_intersect( array_keys( $fields ), array_keys( $_POST ) );
			self::deletePreviousUploads( $_POST['booking_id'], $activeFieldNames );
		}

		// Check if fields are valid before upload starts
		foreach ( $_FILES as $fieldName => $uploaded_file ) {
			$field = $fields[ $fieldName ] ?? null;
			if ( ! $field ) {
				continue; // Skip if field not found in enabled fields
			}

			// File size check
			$maxAllowedFileSize = $field->uploadSize ?? 0;
			if ( $maxAllowedFileSize > 0 && $maxAllowedFileSize < $uploaded_file['size'] ) {

				wp_send_json_error(
					array(
						'error'          => true,
						'fieldWithError' => $fieldName,
						/* translators: %s: Field title. */
						'message'        => sprintf( __( 'The %s file exceeds the upload limit.', 'mpa-checkout-fields' ), $field->title ),
					)
				);
				return;
			}

			// File types
			$supportedFileExtensions = isset( $field->fileTypes ) ? $field->fileTypes : array();
			$allowedFileMimeTypes    = wp_get_mime_types();

			if ( ! empty( $supportedFileExtensions ) ) {

				$allowedFileMimeTypes = array_filter(
					$allowedFileMimeTypes,
					function ( $arrayValue, $arrayKey ) use ( $supportedFileExtensions ) {
						$keys = explode( '|', $arrayKey );
						if ( ! empty( array_intersect( $keys, $supportedFileExtensions ) ) ) {
							return true;
						}
					},
					ARRAY_FILTER_USE_BOTH
				);

				if ( empty( $allowedFileMimeTypes ) || ! in_array( $uploaded_file['type'], $allowedFileMimeTypes ) ) {
					// translators: %1$s is the file type
					$error_message = sprintf( __( 'The %1$s file format is not supported.', 'mpa-checkout-fields' ), $uploaded_file['type'] );

					wp_send_json_error(
						array(
							'error'          => true,
							'fieldWithError' => $fieldName,
							'message'        => $error_message,
						)
					);
					return;
				}
			}
		}

		// Start upload process
		foreach ( $_FILES as $fieldName => $uploaded_file ) {
			$field = $fields[ $fieldName ] ?? null;
			if ( ! $field ) {
				continue; // Skip if field not found in enabled fields
			}

			$generatedFileName = '';

			// Upload process
			$wpHandleUploadOverrides['unique_filename_callback'] = function ( string $dir, string $baseFilename, string $extension ) use ( $fieldName, &$generatedFileName ) {

				$fieldName          = mb_strtolower( $fieldName );
				$uploadingFileName  = 'mpa-' . mb_strtolower( $fieldName );
				$now                = \DateTime::createFromFormat( 'U.u', microtime( true ) );
				$uploadingFileName .= '-' . $now->format( 'mdY-His-u' );

				$index        = 0;
				$fullFileName = '';

				do {

					++$index;
					$fullFileName = $uploadingFileName . "-$index" . $extension;

				} while ( file_exists( $dir . '/' . $fullFileName ) );

				$generatedFileName = $fullFileName;

				return $fullFileName;
			};

			$changeWPUploadFolder = function ( $param ) {

				$mydir         = '/' . self::SECRET_UPLOADS;
				$param['path'] = $param['basedir'] . $mydir;
				$param['url']  = $param['baseurl'] . $mydir;
				return $param;
			};

			add_filter( 'upload_dir', $changeWPUploadFolder );
			$uploadResult = wp_handle_upload( $uploaded_file, $wpHandleUploadOverrides );
			remove_filter( 'upload_dir', $changeWPUploadFolder );

			if ( isset( $uploadResult['error'] ) ) {
				// translators:  %1$s is the error result from the upload.
				$error_message = sprintf( __( 'There was an error with uploading your file: %1$s', 'mpa-checkout-fields' ), $uploadResult['error'] );
				wp_send_json_error(
					array(
						'error'          => true,
						'fieldWithError' => $fieldName,
						'message'        => $error_message,
					)
				);
				return;
			}

			$response[ $fieldName ] = $generatedFileName;
		}

		$response['confirmation_mode'] = $MpappConnfirmationMode;
		wp_send_json_success( $response );
	}

	public static function deletePreviousUploads( $draft_id, $activeFieldNames ) {
		$post = get_post( $draft_id );

		if ( $post->post_type !== 'mpa_booking' || $post->post_status !== 'auto-draft' ) {
			return;
		}

		if ( ! $activeFieldNames ) {
			// Delete previuos on new is set
			foreach ( $_FILES as $fieldName => $uploaded_file ) {
				$meta_value = get_post_meta( $draft_id, '_mpa_checkout_fields_' . $fieldName, true );

				if ( $meta_value ) {
					$fieldName = sanitize_text_field( $fieldName );
					$file_path = self::constructFilePath( $meta_value );

					if ( self::validate_file_path( $file_path ) && file_exists( $file_path ) ) {
						wp_delete_file( $file_path ); // Delete the file
					}
				}
			}
		} else {
			// Delete previuos on new is cleared
			foreach ( $activeFieldNames as $fieldName ) {
				$meta_value = get_post_meta( $draft_id, '_mpa_checkout_fields_' . $fieldName, true );

				if ( $meta_value ) {
					$fieldName = sanitize_text_field( $fieldName );
					$file_path = self::constructFilePath( $meta_value );

					if ( self::validate_file_path( $file_path ) && file_exists( $file_path ) ) {
						wp_delete_file( $file_path ); // Delete the file
					}
				}
			}
		}
	}

	private static function constructFilePath( $meta_value ) {
		$upload_dir = wp_upload_dir();
		$base_dir = $upload_dir['basedir'];
		$mydir = '/' . self::SECRET_UPLOADS;
		return $base_dir . $mydir . '/' . $meta_value;
	}

	public static function validate_file_path( $file_path ) {
		// Check for directory traversal characters
		if ( strpos( $file_path, '..' ) !== false ) {
			return false;
		}

		// Ensure the path is within the WordPress upload directory
		$upload_dir = wp_get_upload_dir();
		$base_dir   = realpath( $upload_dir['basedir'] );

		return strpos( realpath( $file_path ), $base_dir ) === 0;
	}

	public static function processViewUploadedFileRequest() {

		if ( ! isset( $_GET['upload_id'] ) ) {
			return;
		}

		if ( ! is_user_logged_in() ) {
			auth_redirect();
			exit();
		}

		$args = apply_filters(
			'mpa_cf_process_upload_link_args',
			array(
				'upload_id'      => isset( $_GET['upload_id'] ) ? (int) $_GET['upload_id'] : null,
				'appointment_id' => isset( $_GET['appointment_id'] ) ? (int) $_GET['appointment_id'] : null,
			)
		);

		if ( ! $args['appointment_id'] ) {
			return;
		}

		$args['has_access'] = current_user_can( 'read_mpa_checkout_fields' );

		$args['has_access'] = apply_filters( 'mpa_cf_upload_link_has_access', $args['has_access'], $args );

		if ( ! $args['has_access'] ) {
			return;
		}

		$metaValue = get_metadata_by_mid( 'post', $args['upload_id'] );

		if ( ! $metaValue || ! $metaValue->meta_value ) {
			return;
		}

		$file = self::constructFilePath( $metaValue->meta_value );

		if ( ! file_exists( $file ) ) {
			return;
		}

		$mime = wp_check_filetype( $file );

		if ( false === $mime['type'] && function_exists( 'mime_content_type' ) ) {
			$mime['type'] = mime_content_type( $file );
		}

		if ( $mime['type'] ) {
			$mimetype = $mime['type'];
		} else {
			$mimetype = 'image/' . substr( $file, strrpos( $file, '.' ) + 1 );
		}

		nocache_headers();
		header( 'Robots: none' );
		header( 'Content-Type: ' . $mimetype . ';charset=' . mb_detect_encoding( file_get_contents( $file ), mb_list_encodings() ) );
		header( 'Content-Transfer-Encoding: binary' );
		header( 'Expires: 0' );
		header( 'Content-Length: ' . filesize( $file ) );

		if ( empty( $_SERVER['SERVER_SOFTWARE'] ) ||
			false === strpos( sanitize_text_field( wp_unslash( $_SERVER['SERVER_SOFTWARE'] ) ), 'Microsoft-IIS' ) ) {

			header( 'Content-Length: ' . filesize( $file ) );
		}

		$last_modified = gmdate( 'D, d M Y H:i:s', filemtime( $file ) );
		$etag          = '"' . md5( $last_modified ) . '"';

		header( "Last-Modified: $last_modified GMT" );
		header( 'ETag: ' . $etag );
		header( 'Expires: ' . gmdate( 'D, d M Y H:i:s', time() + 100000000 ) . ' GMT' );

		// Support for Conditional GET
		$client_etag = isset( $_SERVER['HTTP_IF_NONE_MATCH'] ) ? sanitize_text_field( wp_unslash( $_SERVER['HTTP_IF_NONE_MATCH'] ) ) : false;

		if ( ! isset( $_SERVER['HTTP_IF_MODIFIED_SINCE'] ) ) {
			$_SERVER['HTTP_IF_MODIFIED_SINCE'] = false;
		}

		$client_last_modified = trim( sanitize_text_field( wp_unslash( $_SERVER['HTTP_IF_MODIFIED_SINCE'] ) ) );
		// If string is empty, return 0. If not, attempt to parse into a timestamp
		$client_modified_timestamp = $client_last_modified ? strtotime( $client_last_modified ) : 0;

		// Make a timestamp for our most recent modification...
		$modified_timestamp = strtotime( $last_modified );

		if ( ( $client_last_modified && $client_etag )
			? ( ( $client_modified_timestamp >= $modified_timestamp ) && ( $client_etag == $etag ) )
			: ( ( $client_modified_timestamp >= $modified_timestamp ) || ( $client_etag == $etag ) )
			) {
			status_header( 304 );
			exit;
		}

		@readfile( $file );
		exit( 0 );
	}

	public static function getUploadedFileLink( int $postId, string $fileUploadFieldName ) {

		global $wpdb;

		$uploadFieldMetaData = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM $wpdb->postmeta WHERE post_id = %d AND meta_key = %s", $postId, '_mpa_checkout_fields_' . $fileUploadFieldName ) );

		$link = '';

		if ( ! empty( $uploadFieldMetaData->meta_id ) && ! empty( $uploadFieldMetaData->meta_value ) ) {

			$link = add_query_arg(
				array(
					'appointment_id' => (int) $postId,
					'upload_id'      => (int) $uploadFieldMetaData->meta_id,
				),
				get_site_url( get_current_blog_id() )
			);
		}

		return $link;
	}

	public static function deleteUploadedCheckoutFieldsFilesOfBooking( int $wpPostId ): void {

		if ( get_post_type( $wpPostId ) != 'mpa_booking' ) {
			return;
		}

		$customFields = mpa_checkout_fields_get_custom_customer_fields();

		if ( empty( $customFields ) ) {
			return;
		}

		foreach ( $customFields as $fieldName => $customField ) {

			if ( 'file_upload' == $customField['type'] ) {

				$fileUploadFieldName = '_mpa_checkout_fields_' . $fieldName;
				$uploadedFileMeta   = get_post_meta( $wpPostId, $fileUploadFieldName );

				foreach ( $uploadedFileMeta as $meta_value ) {
					$fullFilePath = self::constructFilePath( $meta_value );

					if ( self::validate_file_path( $fullFilePath ) && file_exists( $fullFilePath ) ) {
						wp_delete_file( $fullFilePath );
					}
				}
			}
		}
	}

	public static function saveAdminBookingCheckoutFieldsFileUploads( $post_id ) {

		static $did_run = false;

		if ( $did_run ) {
			return;
		}
		$did_run = true;

		global $post;

		// if new add post
		if ( $post !== null ) {
			$post_id = $post->ID;
		}

		// Check the user's permissions.
		if ( ! current_user_can( 'edit_mpa_checkout_fields', $post_id ) ) {
			return $post_id;
		}

		// Handle the file upload
		$wpHandleUploadOverrides = array( 'test_form' => false );
		$errors                  = array();

		foreach ( $_FILES as $fieldName => $uploaded_file ) {

			if ( $uploaded_file['size'] !== 0 ) {

				$generatedFileName = '';

				// Upload process
				$wpHandleUploadOverrides['unique_filename_callback'] = function ( string $dir, string $baseFilename, string $extension ) use ( $fieldName, &$generatedFileName ) {

					$fieldName          = mb_strtolower( $fieldName );
					$uploadingFileName  = 'mpa-' . mb_strtolower( $fieldName );
					$now                = \DateTime::createFromFormat( 'U.u', microtime( true ) );
					$uploadingFileName .= '-' . $now->format( 'mdY-His-u' );

					$index        = 0;
					$fullFileName = '';

					do {
						++$index;
						$fullFileName = $uploadingFileName . "-$index" . $extension;
					} while ( file_exists( $dir . '/' . $fullFileName ) );

					$generatedFileName = $fullFileName;

					return $fullFileName;
				};

				$changeWPUploadFolder = function ( $param ) {
					$mydir         = '/' . self::SECRET_UPLOADS;
					$param['path'] = $param['basedir'] . $mydir;
					$param['url']  = $param['baseurl'] . $mydir;
					return $param;
				};

				add_filter( 'upload_dir', $changeWPUploadFolder );
				$uploadResult = wp_handle_upload( $uploaded_file, $wpHandleUploadOverrides );
				remove_filter( 'upload_dir', $changeWPUploadFolder );

				if ( isset( $uploadResult['file'] ) ) {
					// Delete previous
					$previous_value = get_post_meta( $post_id, '_mpa_checkout_fields_' . $fieldName, true );
					if ( $previous_value ) {
						$previous_file = self::constructFilePath( $previous_value );

						if ( self::validate_file_path( $previous_file ) && file_exists( $previous_file ) ) {
							wp_delete_file( $previous_file ); // Delete
						}
					}

					update_post_meta( $post_id, '_mpa_checkout_fields_' . $fieldName, $generatedFileName );
				} elseif ( $uploadResult['error'] ) {
					// translators: %s is a name of a field, for example 'Your ID photo'
					$errors[] = sprintf( __( 'There was an error uploading the %s file.', 'mpa-checkout-fields' ), $fieldName ) . ' ' . $uploadResult['error'];
					set_transient( 'mpa_checkout_fields_admin_upload_errors', $errors, 30 );
					continue;
				}
			}
		}
	}
}
