<?php

namespace MotoPress\Appointment\Addons\GoogleAnalytics;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}
/**
 * Sending Google Analytics events docs:
 * https://developers.google.com/analytics/devguides/collection/ga4/set-up-ecommerce?hl=en
 * https://developers.google.com/analytics/devguides/collection/ga4/reference/events?hl=en&client_type=gtag
 * https://developers.google.com/analytics/devguides/collection/protocol/ga4/sending-events?client_type=gtag
 * https://developers.google.com/analytics/devguides/collection/protocol/ga4/reference/?client_type=gtag
 */
class GoogleAnalyticsHandler {

	const SETTINGS_TAB_SECTION_SLUG = 'google_analytics';

	const BOOKING_META_GOOGLE_ANALYTICS_CLIENT_ID = 'mpa_google_analytics_client_id';
	const ENTITY_META_GOOGLE_ANALYTICS_EVENT_SENT = 'mpa_google_analytics_event_sent';

	const GOOGLE_ANALYTICS_EVENT_ITEMS_MAX_COUNT = 5;


	public function __construct() {

		add_filter(
			'mpa_integrations_settings_tab_sections',
			function( array $integrationsTabSections ) {

				$integrationsTabSections[ static::SETTINGS_TAB_SECTION_SLUG ] = __( 'Google Analytics', 'mpa-google-analytics' );

				return $integrationsTabSections;
			}
		);

		add_filter(
			'mpa_integrations_section_settings',
			function( array $settingsFields, string $currentSettingsTabSection ) {

				if ( static::SETTINGS_TAB_SECTION_SLUG === $currentSettingsTabSection ) {

					$settingsFields += $this->getSettingsFields();
				}

				return $settingsFields;
			},
			10,
			2
		);

		add_action(
			'wp_head',
			function() {
				$this->echoJSCodeOfGoogleAnalyticsEvents();
			},
			99999 // to make sure we add events code after other plugins echo google tag
		);

		// save google analytics client id to payment to send events later
		if ( ! empty( $_COOKIE['_ga'] ) ) {

			add_action(
				'mpa_new_booking_created',
				function( $entity, $newStatus ) {

					$googleAnalyticsClientId = sanitize_text_field( wp_unslash( $_COOKIE['_ga'] ) );

					// input: GA1.1.1275502704.1684143110
					// get 1275502704.1684143110
					$googleAnalyticsClientId = preg_replace( '/GA\d+\.\d+\./', '', $googleAnalyticsClientId );

					update_post_meta(
						$entity->getId(),
						static::BOOKING_META_GOOGLE_ANALYTICS_CLIENT_ID,
						$googleAnalyticsClientId
					);
				},
				10,
				2
			);
		}

		add_action(
			'mpa_booking_status_changed',
			function( $booking, $newStatus, $oldStatus ) {

				if ( \MotoPress\Appointment\PostTypes\Statuses\BookingStatuses::STATUS_CONFIRMED === $newStatus ) {

					$isEventSent = get_post_meta( $booking->getId(), static::ENTITY_META_GOOGLE_ANALYTICS_EVENT_SENT, true );

					if ( ! $isEventSent ) {

						$payment = null;

						if ( mpapp()->settings()->isPaymentConfirmationMode() ) {

							$payments = $booking->getPayments();

							if ( ! empty( $payments ) ) {
								$payment = $payments[0];
							}
						}

						$this->sendGoogleAnalyticsEvent( 'purchase', $booking, $payment );

						update_post_meta( $booking->getId(), static::ENTITY_META_GOOGLE_ANALYTICS_EVENT_SENT, true );
					}
				}
			},
			10,
			3
		);

		add_action(
			'mpa_payment_status_changed',
			function( $payment, $newStatus, $oldStatus ) {

				$booking = $payment->getBooking();

				if ( \MotoPress\Appointment\PostTypes\Statuses\PaymentStatuses::STATUS_REFUNDED === $newStatus ) {

					$isEventSent = get_post_meta( $payment->getId(), static::ENTITY_META_GOOGLE_ANALYTICS_EVENT_SENT, true );

					if ( ! $isEventSent ) {

						$this->sendGoogleAnalyticsEvent( 'refund', $booking, $payment );

						update_post_meta( $payment->getId(), static::ENTITY_META_GOOGLE_ANALYTICS_EVENT_SENT, true );
					}
				}
			},
			10,
			3
		);
	}


	private function echoJSCodeOfGoogleAnalyticsEvents() {

		$googleTrackingId = MPAGoogleAnalytics::getGoogleAnalyticsMeasurementId();

		if ( empty( $googleTrackingId ) ) {
			return;
		}

		if ( MPAGoogleAnalytics::isEchoGoogleAnalyticsCode() ) {
			?>

			<!-- Google tag (gtag.js) -->
			<?php // phpcs:ignore ?>
			<script async src="https://www.googletagmanager.com/gtag/js?id=<?php echo esc_html( $googleTrackingId ); ?>"></script>
			<script>
				window.dataLayer = window.dataLayer || [];
				function gtag(){dataLayer.push(arguments);}
				gtag('js', new Date());
				gtag('config', '<?php echo esc_html( $googleTrackingId ); ?>');
			</script>

			<?php
		}

		?>
		<script>
			(function() {
				function mpaHtmlSpecialCharsDecode(str) {
					var map = {
						'&amp;': '&',
						'&lt;': '<',
						'&gt;': '>',
						'&quot;': '"',
						'&#039;': "'"
					};

					return str.replace(/&amp;|&lt;|&gt;|&quot;|&#039;/g, function(matched) {
						return map[matched];
					});
				}

				function mpaGetGoogleCartItemData(mpaCartItem) {

					var cartItemData = {
						item_id: mpaCartItem.service.id,
						item_name: mpaCartItem.service.name,
						price: mpaCartItem.getPrice(),
						quantity: 1
					};

					var itemCategoryIndex = 1;
					Object.values(mpaCartItem.serviceCategories).slice(0, 5).forEach((serviceCategoryName) => {

						cartItemData['item_category' + (1 < itemCategoryIndex ? itemCategoryIndex : '')] = mpaHtmlSpecialCharsDecode(serviceCategoryName).substring(0, 100);

					});

					return cartItemData;
				}

				document.addEventListener('mpa_view_item', function(event) {

					gtag('event', 'view_item', {
						currency: event.detail.currencyCode,
						value: event.detail.cartItem.service.price,
						items: [mpaGetGoogleCartItemData(event.detail.cartItem)]
					});
				});

				document.addEventListener('mpa_add_to_cart', function(event) {

					gtag('event', 'add_to_cart', {
						currency: event.detail.currencyCode,
						value: event.detail.cartItem.service.price,
						items: [mpaGetGoogleCartItemData(event.detail.cartItem)]
					});
				});

				document.addEventListener('mpa_view_cart', function(event) {

					let googleEventItems = [];

					event.detail.cart.items.forEach((mpaCartItem) => {

						googleEventItems.push(mpaGetGoogleCartItemData(mpaCartItem));
					});

					gtag('event', 'view_cart', {
						currency: event.detail.currencyCode,
						value: event.detail.cart.getTotalPrice(),
						items: googleEventItems
					});
				});

				document.addEventListener('mpa_remove_from_cart', function(event) {

					gtag('event', 'remove_from_cart', {
						currency: event.detail.currencyCode,
						value: event.detail.cartItem.service.price,
						items: [mpaGetGoogleCartItemData(event.detail.cartItem)]
					});
				});

				document.addEventListener('mpa_begin_checkout', function(event) {

					let googleEventItems = [];

					event.detail.cart.items.forEach((mpaCartItem) => {

						googleEventItems.push(mpaGetGoogleCartItemData(mpaCartItem));
					});

					gtag('event', 'begin_checkout', {
						currency: event.detail.currencyCode,
						value: event.detail.cart.getTotalPrice(),
						items: googleEventItems
					});
				});
			})();
		</script>
		<?php
	}


	private function sendGoogleAnalyticsEvent( string $eventType, \MotoPress\Appointment\Entities\Booking $booking,
		?\MotoPress\Appointment\Entities\Payment $payment = null ) {

		$googleAnalyticsClientId = get_post_meta( $booking->getId(), static::BOOKING_META_GOOGLE_ANALYTICS_CLIENT_ID, true );

		$measurementId = MPAGoogleAnalytics::getGoogleAnalyticsMeasurementId();
		$apiSecret     = MPAGoogleAnalytics::getGoogleAnalyticsApiSecret();

		if ( ! empty( $googleAnalyticsClientId ) &&
			! empty( $measurementId ) && ! empty( $apiSecret ) ) {

			wp_remote_post(
				add_query_arg(
					array(
						'measurement_id' => $measurementId,
						'api_secret'     => $apiSecret,
					),
					'https://www.google-analytics.com/mp/collect'
				),
				array(
					'headers' => array(
						'Content-Type' => 'application/json',
					),
					'body'    => wp_json_encode(
						array(
							'client_id' => $googleAnalyticsClientId,
							'events'    => array(
								array(
									'name'   => $eventType,
									'params' => $this->getGoogleAnalyticsEventParams( $booking, $payment ),
								),
							),
						)
					),
				)
			);
		}
	}

	private function getGoogleAnalyticsEventParams( $booking, ?\MotoPress\Appointment\Entities\Payment $payment = null ): array {

		$reservations            = $booking->getReservations();
		$eventItemsData          = array();
		$reservationDeposiPrices = null;
		$eventValue              = $booking->getTotalPrice();

		if ( null !== $payment ) {

			$eventValue = $payment->getAmount();
		}

		if ( $eventValue < $booking->getTotalPrice() ) {

			$reservationDeposiPrices = \MotoPress\Appointment\Helpers\PriceCalculationHelper::calculateReservationsDepositPrices( $booking );
		}

		foreach ( $reservations as $reservation ) {

			$service = $reservation->getService();

			if ( null === $service ) {
				continue;
			}

			$eventItemData = array(
				'item_id'   => $service->getId(),
				'item_name' => htmlspecialchars_decode( $service->getTitle() ),
				'quantity'  => 1,
				'price'     => null === $reservationDeposiPrices ? $reservation->getPrice() : $reservationDeposiPrices[ $reservation->getId() ],
				'currency'  => mpapp()->settings()->getCurrency(),
			);

			$eventItemData = array_merge(
				$eventItemData,
				$this->getGoogleAnalyticsItemCategoriesData( $service->getCategories() )
			);

			$eventItemsData[] = $eventItemData;
		}

		$eventData = array(
			'currency'       => mpapp()->settings()->getCurrency(),
			'value'          => $eventValue,
			'transaction_id' => null !== $payment ? $payment->getId() : $booking->getId(),
			'items'          => $eventItemsData,
		);

		$coupon = $booking->getCoupon();

		if ( null !== $coupon ) {

			$eventData['coupon'] = $coupon->getCode();
		}

		return $eventData;
	}

	/**
	 * Return array with maximum allowed count of item categories.
	 * @param WP_Term[] $serviceCategories
	 * @return array [ item_category => item_category_name_1, ... item_category_x => item_category_name_x ]
	 */
	private function getGoogleAnalyticsItemCategoriesData( array $serviceCategories ): array {

		$eventItemCategoriesData = array();

		if ( ! empty( $serviceCategories ) ) {

			$itemCategoryNumber = 1;

			foreach ( $serviceCategories as $serviceCategory ) {

				$eventItemCategoriesData[ 'item_category' . ( 1 < $itemCategoryNumber ? $itemCategoryNumber : '' ) ] = substr( htmlspecialchars_decode( $serviceCategory->name ), 0, 100 );

				$itemCategoryNumber++;

				if ( static::GOOGLE_ANALYTICS_EVENT_ITEMS_MAX_COUNT < $itemCategoryNumber ) {
					break;
				}
			}
		}

		return $eventItemCategoriesData;
	}

	private function getSettingsFields(): array {

		$settingsFields = array(
			'mpa_google_analytics_settings_group'        => array(
				'type'  => 'group',
				'label' => __( 'Google Analytics', 'mpa-google-analytics' ),
			),
			MPAGoogleAnalytics::SETTINGS_NAME_MEASUREMENT_ID => array(
				'type'         => 'text',
				'label'        => __( 'Measurement ID', 'mpa-google-analytics' ),
				'description'  => __( 'To obtain the Google Analytics Measurement ID, sign up or sign in to your Google Analytics account. Then, create a property for your website, add a data stream, and navigate to: Admin > Data Streams > select your stream > Measurement ID. Copy the provided Measurement ID from there.', 'mpa-google-analytics' ),
				'size'         => 'regular',
				'default'      => MPAGoogleAnalytics::getGoogleAnalyticsMeasurementId(),
				'translatable' => false,
			),
			MPAGoogleAnalytics::SETTINGS_NAME_IS_ECHO_GOOGLE_ANALYTICS_CODE => array(
				'type'        => 'checkbox',
				'label'       => __( 'Google Tag', 'mpa-google-analytics' ),
				'label2'      => __( 'Add the Google Analytics code to the website header.', 'mpa-google-analytics' ),
				'description' => __( 'Do not select this checkbox if you have already added Google Analytics code to your site through another plugin or your site code.', 'mpa-google-analytics' ),
				'default'     => MPAGoogleAnalytics::isEchoGoogleAnalyticsCode(),
			),
			MPAGoogleAnalytics::SETTINGS_NAME_API_SECRET => array(
				'type'         => 'text',
				'label'        => __( 'API Secret', 'mpa-google-analytics' ),
				'description'  => __( 'An API Secret generated in the Google Analytics UI. To create a new secret, navigate to: Admin > Data Streams > choose your stream > Measurement Protocol > Create.', 'mpa-google-analytics' ),
				'size'         => 'regular',
				'default'      => MPAGoogleAnalytics::getGoogleAnalyticsApiSecret(),
				'translatable' => false,
			),
		);

		if ( MPAGoogleAnalytics::isEDDLicenseEnabled() ) {

			$settingsFields[ MPAGoogleAnalytics::SETTINGS_NAME_PLUGIN_LICENSE_KEY ] = array(
				'type'        => 'mpa-google-analytics-license-settings',
				'label'       => __( 'License Key', 'mpa-google-analytics' ),
				'description' => __( "The License Key is required in order to get automatic plugin updates and support. You can manage your License Key in your personal account. <a href='https://motopress.zendesk.com/hc/en-us/articles/202812996-How-to-use-your-personal-MotoPress-account' target='blank'>Learn more</a>.", 'mpa-google-analytics' ),
				'size'        => 'regular',
			);
		}

		return $settingsFields;
	}
}
