<?php

namespace MotoPress\Appointment\Addons\GoogleAnalytics;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}


class PluginLifecycleHandler {

	private const REQUIRED_APPOINTMENT_PLUGIN_VERSION = '1.17.2';

	private const OPTION_NAME_IS_PLUGIN_ACTIVATED     = 'mpa_google_analytics_activated';
	private const OPTION_NAME_UPGRADED_PLUGIN_VERSION = 'mpa_google_analytics_upgraded_plugin_version';

	private $pluginMainFilePath = '';

	private $isWPEnvironmentSuitedForPlugin = true;


	public function __construct( string $pluginMainFilePath ) {

		$this->pluginMainFilePath = $pluginMainFilePath;

		add_action(
			'plugins_loaded',
			function() {
				$this->checkIsWPEnvironmentSuitedForPlugin();
			},
			-1
		);

		register_activation_hook(
			$this->pluginMainFilePath,
			function( $isNetworkWide = false ) {
				$this->activatePlugin( $isNetworkWide );
			}
		);

		// add installation for a new site in multisite WordPress
		add_action(
			version_compare( get_bloginfo( 'version' ), '5.1', '>=' ) ? 'wp_initialize_site' : 'wpmu_new_blog',
			/**
			 * @param $blog in case of wp_initialize_site action is WP_Site otherwise int (site id)
			 */
			function( $blog ) {
				$blogId = is_int( $blog ) ? $blog : $blog->blog_id;
				$this->activatePlugin( false, $blogId );
			}
		);

		register_deactivation_hook(
			$this->pluginMainFilePath,
			function() {
				$this->deactivatePlugin();
			}
		);

		register_uninstall_hook( $this->pluginMainFilePath, array( __CLASS__, 'uninstallPlugin' ) );

		// initialize EDD updater
		if ( ! wp_doing_ajax() && ! wp_doing_cron() ) {

			add_action(
				'admin_init',
				function() {

					if ( MPAGoogleAnalytics::isEDDLicenseEnabled() ) {

						new EDDPluginUpdater(
							MPAGoogleAnalytics::getPluginSourceServerUrl(),
							$this->pluginMainFilePath,
							array(
								'version' => MPAGoogleAnalytics::getPluginVersion(),
								'license' => MPAGoogleAnalytics::getLicenseKey(),
								'item_id' => MPAGoogleAnalytics::getProductId(),
								'author'  => MPAGoogleAnalytics::getPluginAuthorName(),
							)
						);
					}
				}
			);
		}

		// check and upgrade plugin data if it is needed after plugin update
		add_action(
			'init',
			function() {
				if ( static::isPluginActivated() && $this->isWPEnvironmentSuitedForPlugin() ) {

					$this->upgradePluginData();
				}
			}
		);

	}


	private function checkIsWPEnvironmentSuitedForPlugin(): void {

		$isRequiredVersionOfAppointmentPluginActive = false;

		if ( defined( 'MotoPress\Appointment\VERSION' ) ) {

			$isRequiredVersionOfAppointmentPluginActive = version_compare(
				\MotoPress\Appointment\VERSION,
				static::REQUIRED_APPOINTMENT_PLUGIN_VERSION,
				'>='
			);
		}

		if ( ! $isRequiredVersionOfAppointmentPluginActive ) {

			$this->isWPEnvironmentSuitedForPlugin = false;

			global $pagenow;

			if ( isset( $pagenow ) && 'plugins.php' === $pagenow ) {

				$this->addErrorAdminNotice(
					sprintf(
						// translators: %1$s is the name of the required plugin, %2$s is the required plugin version like 1.x.x
						esc_html__( 'The Google Analytics for Appointment Booking addon requires the %1$s plugin version %2$s or higher. Install and activate it for proper work.', 'mpa-google-analytics' ),
						'Appointment Booking',
						esc_html( static::REQUIRED_APPOINTMENT_PLUGIN_VERSION )
					)
				);
			}
		}
	}

	private function addErrorAdminNotice( string $errorText ): void {

		add_action(
			'admin_notices',
			function() use ( $errorText ) {
				echo '<div class="notice notice-warning">
						<p>' . esc_html( $errorText ) . '</p>
					</div>';
			}
		);
	}


	public function isWPEnvironmentSuitedForPlugin(): bool {

		return $this->isWPEnvironmentSuitedForPlugin;
	}


	private static function isPluginActivated(): bool {
		return (bool) get_option( static::OPTION_NAME_IS_PLUGIN_ACTIVATED, false );
	}


	private static function setPluginActivated( $activated = true ): void {
		update_option( static::OPTION_NAME_IS_PLUGIN_ACTIVATED, $activated, false );
	}


	private function activatePlugin( $isNetworkWide = false, $multisiteBlogId = 0 ) {

		// check environment because plugin was not loaded yet before activation
		if ( ! $this->checkIsWPEnvironmentSuitedForPlugin() ) {

			return;
		}

		if ( is_multisite() && $isNetworkWide ) {

			$sites = get_sites();

			foreach ( $sites as $site ) {

				switch_to_blog( $site->blog_id );

				$this->installPlugin();

				restore_current_blog();
			}
		} elseif ( is_multisite() && 0 < $multisiteBlogId ) {

			if ( ! is_plugin_active_for_network( plugin_basename( $this->pluginMainFilePath ) ) ) {
				return;
			}

				switch_to_blog( $multisiteBlogId );

				$this->installPlugin();

				restore_current_blog();

		} else {

			$this->installPlugin();
		}
	}

	private function installPlugin() {

		if ( static::isPluginActivated() ) {
			return;
		}

		// put installation code here...

		$this->upgradePluginData();

		static::setPluginActivated();
	}

	private function upgradePluginData() {

		// put start plugin patch code here...
	}


	private function deactivatePlugin() {}

	/**
	 * Do not use any plugin classed here to make sure that any plugin's
	 * code will not be working during uninstallation.
	 */
	public static function uninstallPlugin() {

		// put uninstall code here...

		update_option( static::OPTION_NAME_UPGRADED_PLUGIN_VERSION, null );
		static::setPluginActivated( false );
	}
}
