<?php
/**
 * Plugin Name: Google Analytics for Appointment Booking
 * Plugin URI: https://motopress.com/products/appointment-booking-google-analytics/
 * Description: Integrates with Google Analytics to track e-commerce data associated with Appointment Booking.
 * Version: 1.0.1
 * Requires at least: 5.7
 * Requires PHP: 7.2
 * Author: MotoPress
 * Author URI: https://motopress.com/
 * License: GPLv2 or later
 * Text Domain: mpa-google-analytics
 * Domain Path: /languages
 */

namespace MotoPress\Appointment\Addons\GoogleAnalytics;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

final class MPAGoogleAnalytics {

	public const SETTINGS_NAME_MEASUREMENT_ID                = 'mpa_google_analytics_measurement_id';
	public const SETTINGS_NAME_IS_ECHO_GOOGLE_ANALYTICS_CODE = 'mpa_is_echo_google_analytics_code';
	public const SETTINGS_NAME_API_SECRET                    = 'mpa_google_analytics_api_secret';
	public const SETTINGS_NAME_PLUGIN_LICENSE_KEY            = 'mpa_google_analytics_license_key';

	private static $pluginClasses = array(
		PluginLicenseSettingsField::class => 'includes/fields/plugin-license-settings-field.php',
		PluginLifecycleHandler::class     => 'includes/handlers/plugin-lifecycle-handler.php',
		EDDPluginUpdater::class           => 'includes/handlers/edd-plugin-updater.php',
		GoogleAnalyticsHandler::class     => 'includes/handlers/google-analytics-handler.php',
	);

	private static $instance;

	private $pluginData;
	private $pluginDir;
	private $pluginUrl;

	// prevent cloning of singleton
	public function __clone() {}
	public function __wakeup() {}

	public static function getInstance() {

		if ( ! isset( self::$instance ) ) {

			self::$instance = new MPAGoogleAnalytics();
			// init plugin in a separate method to execute some plugin methods
			self::$instance->initPlugin();
		}

		return self::$instance;
	}

	private function __construct() {

		$this->pluginDir = plugin_dir_path( __FILE__ );
		$this->pluginUrl = plugin_dir_url( __FILE__ );

		spl_autoload_register(
			function( $class ) {
				$this->loadPluginClass( $class );
			},
			true,
			true
		);

		// load dependencies from vendor folder
		// require_once $this->pluginDir . 'vendor/autoload.php';
	}

	private function initPlugin(): void {

		$pluginLifecycleHandler = new PluginLifecycleHandler( __FILE__ );

		add_action(
			'plugins_loaded',
			function() use ( $pluginLifecycleHandler ) {

				load_plugin_textdomain(
					'mpa-google-analytics',
					false,
					$this->pluginDir . 'languages/'
				);

				// do not initialize plugin if environment does not suite it
				if ( ! $pluginLifecycleHandler->isWPEnvironmentSuitedForPlugin() ) {
					return;
				}

				// register plugins custom settings field
				add_filter(
					'mpa_' . PluginLicenseSettingsField::TYPE . '_field_class',
					function() {
						return PluginLicenseSettingsField::class;
					}
				);

				PluginLicenseSettingsField::registerField();

				new GoogleAnalyticsHandler();
			},
			16 // init after Appointment Booking
		);
	}


	private function loadPluginClass( string $classNameWithNamespace ) {

		$classNameWithNamespace = ltrim( $classNameWithNamespace, '\\' );

		if ( ! str_starts_with( $classNameWithNamespace, __NAMESPACE__ ) ) {
			return;
		}

		if ( ! empty( static::$pluginClasses[ $classNameWithNamespace ] ) ) {

			$classFilePath = static::getPluginPath( static::$pluginClasses[ $classNameWithNamespace ] );

			if ( file_exists( $classFilePath ) ) {

				require_once $classFilePath;
			}
		}
	}

	/**
	 * @param string $relativeFilePath - starts without /
	 */
	public static function getPluginPath( string $relativeFilePath = '' ): string {

		return static::getInstance()->pluginDir . $relativeFilePath;
	}

	/**
	 * @param string $relativeUrl - starts without /
	 */
	public static function getPluginUrl( string $relativeUrl = '' ): string {

		return static::getInstance()->pluginUrl . $relativeUrl;
	}

	public static function getProductId(): string {
		return '1452392';
	}

	private static function getPluginData(): array {
		if ( empty( static::getInstance()->pluginData ) ) {
			static::getInstance()->pluginData = get_plugin_data( __FILE__, false, false );
		}
		return static::getInstance()->pluginData;
	}

	public static function getPluginAuthorName(): string {
		$pluginData = static::getPluginData();
		return isset( $pluginData['Author'] ) ? $pluginData['Author'] : '';
	}

	public static function getPluginVersion(): string {
		$pluginData = static::getPluginData();
		return isset( $pluginData['Version'] ) ? $pluginData['Version'] : '';
	}

	public static function getPluginSourceServerUrl(): string {
		$pluginData = static::getPluginData();
		return isset( $pluginData['PluginURI'] ) ? $pluginData['PluginURI'] : '';
	}


	public static function isEDDLicenseEnabled(): bool {
		return (bool) apply_filters( 'mpa_google_analytics_use_edd_license', true );
	}

	public static function getLicenseKey(): string {
		return get_option( static::SETTINGS_NAME_PLUGIN_LICENSE_KEY, '' );
	}

	public static function getGoogleAnalyticsMeasurementId(): string {
		return get_option( static::SETTINGS_NAME_MEASUREMENT_ID, '' );
	}

	public static function isEchoGoogleAnalyticsCode(): bool {
		return boolval( get_option( static::SETTINGS_NAME_IS_ECHO_GOOGLE_ANALYTICS_CODE, false ) );
	}

	public static function getGoogleAnalyticsApiSecret(): string {
		return get_option( static::SETTINGS_NAME_API_SECRET, '' );
	}
}

// init plugin
MPAGoogleAnalytics::getInstance();
