<?php

namespace MotoPress\Appointment\Invoices\Admin;

use MotoPress\Appointment\Fields\AbstractField;
use MotoPress\Appointment\Invoices\Plugin;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class PluginLicenseSettingsField extends AbstractField {

	const TYPE = 'invoices-license-settings';

	protected $default = '';

	public static function registerField() {

		add_action(
			'admin_init',
			function () {
				if ( isset( $_POST['mpa_invoices_edd_nonce'] ) && isset( $_POST['edd_license_activate'] ) ) {

					if ( ! check_admin_referer( 'mpa_invoices_edd_nonce', 'mpa_invoices_edd_nonce' ) ) {
						return;
					}

					static::activateLicense();

				} elseif ( isset( $_POST['mpa_invoices_edd_nonce'] ) && isset( $_POST['edd_license_deactivate'] ) ) {

					if ( ! check_admin_referer( 'mpa_invoices_edd_nonce', 'mpa_invoices_edd_nonce' ) ) {
						return;
					}

					static::deactivateLicense();
				}
			}
		);
	}

	protected function validateValue( $value ) {
		return sanitize_text_field( $value );
	}

	public function renderInput() {

		$licenseKey = Plugin::getInstance()->getEddLicenseApi()->getKey();

		if ( $licenseKey ) {

			$licenseData = $this->getLicenseDataFromRemoteServer();

		}

		$licenseStatus = '';

		if ( isset( $licenseData, $licenseData->license ) ) {

			switch ( $licenseData->license ) {

				case 'inactive':
				case 'site_inactive':
					$licenseStatus = esc_html__( 'Inactive', 'mpa-invoices' );
					break;

				case 'valid':
					if ( 'lifetime' !== $licenseData->expires ) {

						$date          = ( $licenseData->expires ) ? new \DateTime( $licenseData->expires ) : false;
						$expires       = ( $date ) ? ' ' . $date->format( 'd.m.Y' ) : '';
						$licenseStatus = esc_html__( 'Valid until', 'mpa-invoices' ) . $expires;

					} else {

						$licenseStatus = esc_html__( 'Valid (Lifetime)', 'mpa-invoices' );
					}
					break;

				case 'disabled':
					$licenseStatus = esc_html__( 'Disabled', 'mpa-invoices' );
					break;

				case 'expired':
					$licenseStatus = esc_html__( 'Expired', 'mpa-invoices' );
					break;

				case 'invalid':
					$licenseStatus = esc_html__( 'Invalid', 'mpa-invoices' );
					break;

				case 'item_name_mismatch':
					$licenseStatus = mpa_kses_link( __( "Your License Key does not match the installed plugin. <a href='https://motopress.zendesk.com/hc/en-us/articles/202957243-What-to-do-if-the-license-key-doesn-t-correspond-with-the-plugin-license' target='_blank'>How to fix this.</a>", 'mpa-invoices' ) );
					break;

				case 'invalid_item_id':
					$licenseStatus = esc_html__( 'Product ID is not valid', 'mpa-invoices' );
					break;
			}

			$licenseStatus = esc_html__( 'Status', 'mpa-invoices' ) . ': ' . $licenseStatus;
		}

		$output = '';

		ob_start();

		?>

		<div>
			<input id="mpa_invoices_edd_license_key" name="<?php echo esc_attr( $this->inputName ); ?>" type="text"
				class="regular-text" value="<?php echo esc_attr( $licenseKey ); ?>"/>

			<p>
			<?php if ( $licenseKey ) : ?>

				<p><i><?php echo esc_html( str_repeat( '&#8226;', 20 ) . substr( $licenseKey, -7 ) ); ?></i></p>
				<p><strong><?php echo esc_html( $licenseStatus ); ?></strong></p>

			<?php endif; ?>

			<?php
			if ( isset( $licenseData, $licenseData->license ) &&
				in_array( strval( $licenseData->license ), array( 'inactive', 'site_inactive', 'valid', 'expired' ), true ) ) :
				?>

				<br>
				<p>

					<?php
					if ( 'inactive' === $licenseData->license || 'site_inactive' === $licenseData->license ) :

						wp_nonce_field( 'mpa_invoices_edd_nonce', 'mpa_invoices_edd_nonce' );
						?>
						<input type="submit" class="button-secondary" name="edd_license_activate"
							value="<?php esc_html_e( 'Activate License', 'mpa-invoices' ); ?>"/>

					<?php elseif ( 'valid' === $licenseData->license ) : ?>

						<?php wp_nonce_field( 'mpa_invoices_edd_nonce', 'mpa_invoices_edd_nonce' ); ?>

						<input type="submit" class="button-secondary" name="edd_license_deactivate"
							value="<?php esc_html_e( 'Deactivate License', 'mpa-invoices' ); ?>"/>

					<?php elseif ( 'expired' === $licenseData->license ) : ?>

						<a href="<?php echo esc_attr( mpapp()->settings()->getRenewUrl() ); ?>" class="button-secondary" target="_blank">
							<?php esc_html_e( 'Renew License', 'mpa-invoices' ); ?>
						</a>

					<?php endif; ?>

				</p>
			<?php endif; ?>
		</div>

		<?php if ( $licenseKey ) : ?>
			<br>
		<?php endif; ?>

		<?php

		$output = ob_get_contents();
		if ( $output ) {
			ob_end_clean();
		}

		return $output;
	}

	/**
	 * @return stdClass|null
	 */
	private function getLicenseDataFromRemoteServer() {
		return Plugin::getInstance()->getEddLicenseApi()->check();
	}

	/**
	 * @return stdClass|false
	 */
	private static function activateLicense() {
		return Plugin::getInstance()->getEddLicenseApi()->activate();
	}

	/**
	 * @return stdClass|false
	 */
	private static function deactivateLicense() {
		return Plugin::getInstance()->getEddLicenseApi()->deactivate();
	}
}
