<?php

namespace MotoPress\Appointment\Addons\SquarePayments;

use MotoPress\Appointment\Fields\AbstractField;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}


class PluginLicenseSettingsField extends AbstractField {

	const TYPE = 'mpa-square-payments-license-settings';

	protected $default = '';

	public static function registerField() {

		add_action(
			'admin_init',
			function() {
				if ( isset( $_POST['mpasp_edd_nonce'] ) && isset( $_POST['edd_license_activate'] ) ) {

					if ( ! check_admin_referer( 'mpasp_edd_nonce', 'mpasp_edd_nonce' ) ) {
						return;
					}

					static::activateLicense();

				} elseif ( isset( $_POST['mpasp_edd_nonce'] ) && isset( $_POST['edd_license_deactivate'] ) ) {

					if ( ! check_admin_referer( 'mpasp_edd_nonce', 'mpasp_edd_nonce' ) ) {
						return;
					}

					static::deactivateLicense();
				}
			}
		);
	}

	protected function validateValue( $value ) {
		return sanitize_text_field( $value );
	}

	public function renderInput() {

		$licenseKey = MPASquarePayments::getLicenseKey();

		if ( $licenseKey ) {

			$licenseData = $this->getLicenseDataFromRemoteServer();
		}

		$licenseStatus = '';

		if ( isset( $licenseData, $licenseData->license ) ) {

			switch ( $licenseData->license ) {

				case 'inactive':
				case 'site_inactive':
					$licenseStatus = esc_html__( 'Inactive', 'mpa-square-payments' );
					break;

				case 'valid':
					if ( 'lifetime' !== $licenseData->expires ) {

						$date          = ( $licenseData->expires ) ? new \DateTime( $licenseData->expires ) : false;
						$expires       = ( $date ) ? ' ' . $date->format( 'd.m.Y' ) : '';
						$licenseStatus = esc_html__( 'Valid until', 'mpa-square-payments' ) . $expires;

					} else {

						$licenseStatus = esc_html__( 'Valid (Lifetime)', 'mpa-square-payments' );
					}
					break;

				case 'disabled':
					$licenseStatus = esc_html__( 'Disabled', 'mpa-square-payments' );
					break;

				case 'expired':
					$licenseStatus = esc_html__( 'Expired', 'mpa-square-payments' );
					break;

				case 'invalid':
					$licenseStatus = esc_html__( 'Invalid', 'mpa-square-payments' );
					break;

				case 'item_name_mismatch':
					$licenseStatus = mpa_kses_link( __( "Your License Key does not match the installed plugin. <a href='https://motopress.zendesk.com/hc/en-us/articles/202957243-What-to-do-if-the-license-key-doesn-t-correspond-with-the-plugin-license' target='_blank'>How to fix this.</a>", 'mpa-square-payments' ) );
					break;

				case 'invalid_item_id':
					$licenseStatus = esc_html__( 'Product ID is not valid', 'mpa-square-payments' );
					break;
			}

			$licenseStatus = esc_html__( 'Status', 'mpa-square-payments' ) . ': ' . $licenseStatus;
		}

		$output = '';

		ob_start();

		?>

		<div>
			<input id="mpa_square_payments_license_key" name="<?php echo esc_attr( $this->inputName ); ?>" type="text"
				class="regular-text" value="<?php echo esc_attr( $licenseKey ); ?>"/>

			<p>
			<?php if ( $licenseKey ) : ?>

				<p><i><?php echo esc_html( str_repeat( '&#8226;', 20 ) . substr( $licenseKey, -7 ) ); ?></i></p>
				<p><strong><?php echo esc_html( $licenseStatus ); ?></strong></p>

			<?php endif; ?>

			<?php
			if ( isset( $licenseData, $licenseData->license ) &&
				in_array( strval( $licenseData->license ), array( 'inactive', 'site_inactive', 'valid', 'expired' ), true ) ) :
				?>

				<br>
				<p>

					<?php
					if ( 'inactive' === $licenseData->license || 'site_inactive' === $licenseData->license ) :

						wp_nonce_field( 'mpasp_edd_nonce', 'mpasp_edd_nonce' );
						?>
						<input type="submit" class="button-secondary" name="edd_license_activate"
							value="<?php esc_html_e( 'Activate License', 'mpa-square-payments' ); ?>"/>

					<?php elseif ( 'valid' === $licenseData->license ) : ?>

						<?php wp_nonce_field( 'mpasp_edd_nonce', 'mpasp_edd_nonce' ); ?>

						<input type="submit" class="button-secondary" name="edd_license_deactivate"
							value="<?php esc_html_e( 'Deactivate License', 'mpa-square-payments' ); ?>"/>

					<?php elseif ( 'expired' === $licenseData->license ) : ?>

						<a href="<?php echo esc_attr( mpapp()->settings()->getRenewUrl() ); ?>" class="button-secondary" target="_blank">
							<?php esc_html_e( 'Renew License', 'mpa-square-payments' ); ?>
						</a>

					<?php endif; ?>

				</p>
			<?php endif; ?>
		</div>

		<?php if ( $licenseKey ) : ?>
			<br>
		<?php endif; ?>

		<?php

		$output = ob_get_contents();
		if ( $output ) {
			ob_end_clean();
		}

		return $output;
	}

	/**
	 * @return stdClass|null
	 */
	private function getLicenseDataFromRemoteServer() {

		$checkLicenseUrl = add_query_arg(
			array(
				'edd_action' => 'check_license',
				'license'    => MPASquarePayments::getLicenseKey(),
				'item_id'    => MPASquarePayments::getProductId(),
				'url'        => home_url(),
			),
			MPASquarePayments::getPluginSourceServerUrl()
		);

		$response = wp_remote_get(
			$checkLicenseUrl,
			array(
				'timeout'   => 15,
				'sslverify' => false,
			)
		);

		if ( is_wp_error( $response ) ) {
			return null;
		}

		return json_decode( wp_remote_retrieve_body( $response ) );
	}

	/**
	 * @return stdClass|false
	 */
	private static function activateLicense() {

		$activateUrl = add_query_arg(
			array(
				'edd_action' => 'activate_license',
				'license'    => MPASquarePayments::getLicenseKey(),
				'item_id'    => MPASquarePayments::getProductId(),
				'url'        => home_url(),
			),
			MPASquarePayments::getPluginSourceServerUrl()
		);

		$response = wp_remote_get(
			$activateUrl,
			array(
				'timeout'   => 15,
				'sslverify' => false,
			)
		);

		if ( is_wp_error( $response ) ) {
			return false;
		}

		$licenseData = json_decode( wp_remote_retrieve_body( $response ) );

		return $licenseData;
	}

	/**
	 * @return stdClass|false
	 */
	private static function deactivateLicense() {

		$deactivateUrl = add_query_arg(
			array(
				'edd_action' => 'deactivate_license',
				'license'    => MPASquarePayments::getLicenseKey(),
				'item_id'    => MPASquarePayments::getProductId(),
				'url'        => home_url(),
			),
			MPASquarePayments::getPluginSourceServerUrl()
		);

		$response = wp_remote_get(
			$deactivateUrl,
			array(
				'timeout'   => 15,
				'sslverify' => false,
			)
		);

		if ( is_wp_error( $response ) ) {
			return false;
		}

		$licenseData = json_decode( wp_remote_retrieve_body( $response ) );

		return $licenseData;
	}
}
