<?php

namespace MotoPress\Appointment\Addons\SquarePayments;

use MotoPress\Appointment\Payments\Gateways\AbstractPaymentGateway;
use MotoPress\Appointment\Entities\Payment;
use \MotoPress\Appointment\Entities\Booking;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Web client SDK doc: https://developer.squareup.com/docs/web-payments/overview
 * Web client SDK examples: https://developer.squareup.com/reference/sdks/web/payments
 * Backend API SDK doc: https://developer.squareup.com/docs/sdks/php
 * Test cards and so on: https://developer.squareup.com/docs/devtools/sandbox/payments
 */
class SquarePaymentsGateway extends AbstractPaymentGateway {

	const WEB_SDK_SCRIPT_URL_PROD = 'https://web.squarecdn.com/v1/square.js';
	const WEB_SDK_SCRIPT_URL_TEST = 'https://sandbox.web.squarecdn.com/v1/square.js';

	const UNPREFIXED_SETTING_NAME_APPLICATION_ID          = 'application_id';
	const UNPREFIXED_SETTING_NAME_ACCESS_TOKEN            = 'access_token';
	const UNPREFIXED_SETTING_NAME_LOCATION_ID             = 'location_id';
	const UNPREFIXED_SETTING_NAME_ENABLED_PAYMENT_METHODS = 'enabled_payment_methods';

	const UNPREFIXED_SETTING_NAME_PLUGIN_LICENSE_KEY = 'plugin_license_key';


	public function getId(): string {
		return 'square';
	}

	public function getName(): string {
		return __( 'Square Payments', 'mpa-square-payments' );
	}

	protected function getDefaultPublicName(): string {
		return __( 'Pay by Square', 'mpa-square-payments' );
	}

	public function isSupportsSandbox(): bool {
		return true;
	}

	public function isActive() {

		return $this->isEnabled() &&
			! empty( $this->getOption( static::UNPREFIXED_SETTING_NAME_APPLICATION_ID ) ) &&
			! empty( $this->getOption( static::UNPREFIXED_SETTING_NAME_ACCESS_TOKEN ) ) &&
			! empty( $this->getOption( static::UNPREFIXED_SETTING_NAME_LOCATION_ID ) ) &&
			! empty( $this->getOption( static::UNPREFIXED_SETTING_NAME_ENABLED_PAYMENT_METHODS ) );
	}

	/**
	 * Fields of payment gateway settings
	 */
	public function getFields(): array {

		$fields = parent::getFields();

		$fields[ $this->getOptionNameRaw( 'sandbox' ) ]['description'] .= sprintf(
			// translators: %s - url of square website page.
			__( 'See test card numbers here: %s', 'mpa-square-payments' ),
			'<a href="https://developer.squareup.com/docs/devtools/sandbox/payments">https://developer.squareup.com/docs/devtools/sandbox/payments</a>'
		);

		$fields[ $this->getOptionNameRaw( static::UNPREFIXED_SETTING_NAME_APPLICATION_ID ) ] = array(
			'type'        => 'text',
			'label'       => __( 'Application ID', 'mpa-square-payments' ),
			'description' => __( 'The Application ID can be found in the Square App Settings > Credentials.', 'mpa-square-payments' ) . '<br>' .
				sprintf(
					// translators: %s is Square documentation url
					esc_html__( 'Find out how to create a Square App here: %s', 'mpa-square-payments' ),
					'<a href="https://developer.squareup.com/docs/build-basics/access-tokens#get-a-personal-access-token" target="blank">https://developer.squareup.com/docs/build-basics/access-tokens#get-a-personal-access-token</a>'
				),
			'size'        => 'regular',
			'value'       => $this->getOption( static::UNPREFIXED_SETTING_NAME_APPLICATION_ID ),
		);

		$fields[ $this->getOptionNameRaw( static::UNPREFIXED_SETTING_NAME_ACCESS_TOKEN ) ] = array(
			'type'        => 'text',
			'label'       => __( 'Access Token', 'mpa-square-payments' ),
			'description' => __( 'The Access Token can be found in the Square App Settings > Credentials.', 'mpa-square-payments' ),
			'size'        => 'regular',
			'value'       => $this->getOption( static::UNPREFIXED_SETTING_NAME_ACCESS_TOKEN ),
		);

		$fields[ $this->getOptionNameRaw( static::UNPREFIXED_SETTING_NAME_LOCATION_ID ) ] = array(
			'type'        => 'text',
			'label'       => __( 'Location ID', 'mpa-square-payments' ),
			'description' => __( 'The Location ID can be found in the Square App Settings > Locations. Location currency must be the same as currency in General settings of the plugin!', 'mpa-square-payments' ),
			'size'        => 'regular',
			'value'       => $this->getOption( static::UNPREFIXED_SETTING_NAME_LOCATION_ID ),
		);

		$fields[ $this->getOptionNameRaw( static::UNPREFIXED_SETTING_NAME_ENABLED_PAYMENT_METHODS ) ] = array(
			'type'    => 'checklist',
			'label'   => __( 'Payment Methods', 'mpa-square-payments' ),
			'options' => array(
				'card'      => __( 'Card', 'mpa-square-payments' ),
				'gift-card' => __( 'Gift card', 'mpa-square-payments' ),
			),
			'value'   => $this->getOption( static::UNPREFIXED_SETTING_NAME_ENABLED_PAYMENT_METHODS, array() ),
		);

		if ( MPASquarePayments::isEDDLicenseEnabled() ) {

			$fields[ MPASquarePayments::SETTINGS_NAME_PLUGIN_LICENSE_KEY ] = array(
				'type'        => PluginLicenseSettingsField::TYPE,
				'label'       => __( 'License Key', 'mpa-square-payments' ),
				'description' => __( "The License Key is required in order to get automatic plugin updates and support. You can manage your License Key in your personal account. <a href='https://motopress.zendesk.com/hc/en-us/articles/202812996-How-to-use-your-personal-MotoPress-account' target='blank'>Learn more</a>.", 'mpa-square-payments' ),
				'size'        => 'regular',
			);
		}

		return $fields;
	}

	public function enqueueScripts(): void {

		// slient side SDK: https://developer.squareup.com/reference/sdks/web/payments
		wp_enqueue_script(
			'square_web_sdk',
			$this->isSandboxModeEnabled() ? static::WEB_SDK_SCRIPT_URL_TEST : static::WEB_SDK_SCRIPT_URL_PROD,
			array(),
			MPASquarePayments::getPluginVersion(),
			false
		);

		wp_enqueue_script(
			'square_gateway',
			MPASquarePayments::getPluginUrl( '/js/public.min.js' ),
			array( 'square_web_sdk', 'jquery' ),
			MPASquarePayments::getPluginVersion(),
			true
		);

		wp_enqueue_style(
			'square_gateway_styles',
			MPASquarePayments::getPluginUrl( '/css/public.css' ),
			array(),
			MPASquarePayments::getPluginVersion()
		);
	}


	public function getFrontendData() {

		return parent::getFrontendData() + array(
			static::UNPREFIXED_SETTING_NAME_APPLICATION_ID => $this->getOption( static::UNPREFIXED_SETTING_NAME_APPLICATION_ID ),
			static::UNPREFIXED_SETTING_NAME_LOCATION_ID    => $this->getOption( static::UNPREFIXED_SETTING_NAME_LOCATION_ID ),
			static::UNPREFIXED_SETTING_NAME_ENABLED_PAYMENT_METHODS => $this->getOption( static::UNPREFIXED_SETTING_NAME_ENABLED_PAYMENT_METHODS ),
			'country_code'                                 => mpapp()->settings()->getCountry(),
			'currency_code'                                => mpapp()->settings()->getCurrency(),
		);
	}


	public function printBillingFields(): void {

		if ( ! $this->isEnabled() ) {
			return;
		}

		$enabledPaymentMethods = $this->getOption( static::UNPREFIXED_SETTING_NAME_ENABLED_PAYMENT_METHODS );
		$defaultPaymentMethod  = $enabledPaymentMethods[0];

		if ( 1 < count( $enabledPaymentMethods ) ) {

			echo '<ul class="mpa-square-payments__methods-list">';

			if ( in_array( 'card', $enabledPaymentMethods, true ) ) {

				echo '<li><label><input type="radio" name="mpa-square-payments__method-switcher" value="card" checked="checked">' .
					esc_html__( 'Card', 'mpa-square-payments' ) . '</label></li>';
			}

			if ( in_array( 'gift-card', $enabledPaymentMethods, true ) ) {

				echo '<li><label><input type="radio" name="mpa-square-payments__method-switcher" value="gift-card">' .
					esc_html__( 'Gift card', 'mpa-square-payments' ) . '</label></li>';
			}

			echo '</ul>';
		}
		?>

		<?php if ( in_array( 'card', $enabledPaymentMethods, true ) ) : ?>
			<div class="mpa-square-payments__method mpa-square-payments__card<?php echo 'card' === $defaultPaymentMethod ? '' : esc_attr( ' mpa-hide' ); ?>"></div>
		<?php endif; ?>

		<?php if ( in_array( 'gift-card', $enabledPaymentMethods, true ) ) : ?>
			<div class="mpa-square-payments__method mpa-square-payments__gift-card<?php echo 'gift-card' === $defaultPaymentMethod ? '' : esc_attr( ' mpa-hide' ); ?>"></div>
		<?php endif; ?>

		<?php
	}

	/**
	 * Creates pending payment transaction which will be processed later.
	 * @param array $paymentData - can contains gateway specific data from frontend
	 * (for example, payment transaction id, token, payment intent id and so on)
	 * @return mixed any gateway specific data needed on frontend
	 */
	public function startPayment( Booking $booking, string $currencyCode, float $payingAmount, array $paymentData ) {

		parent::startPayment( $booking, $currencyCode, $payingAmount, $paymentData );
		return '';
	}

	/**
	 * Each gateway can add here additional payment data for a new starting payment.
	 * @throws \Exception if payment had not been stored
	 */
	protected function prepareAndStoreNewPayment( Payment $payment, Booking $booking, array $paymentData ): Payment {

		$paymentMethod = sanitize_text_field( $paymentData['payment_method'] );
		$payment->setPaymentMethod( $paymentMethod );

		$paymentToken = sanitize_text_field( $paymentData['payment_token'] );
		$payment->setTransactionId( $paymentToken );

		$payment = parent::prepareAndStoreNewPayment( $payment, $booking, $paymentData );

		return $payment;
	}

	/**
	 * @param Payment $payment
	 * @param array $paymentData[ 'booking' => Booking, ... any gateway specific data from frontend ]
	 * @return Payment
	 * @throws \Exception if something goes wrong
	 */
	public function processPayment( $payment, $paymentData ) {

		try {

			$squareClient = new \Square\SquareClient(
				array(
					'accessToken'     => $this->getOption( static::UNPREFIXED_SETTING_NAME_ACCESS_TOKEN ),
					'environment'     => $this->isSandboxModeEnabled() ? \Square\Environment::SANDBOX : \Square\Environment::PRODUCTION,
					'numberOfRetries' => 3,
					'timeout'         => 20, // seconds
				)
			);

			$paymentsApi = $squareClient->getPaymentsApi();

			$money = new \Square\Models\Money();
			// Monetary amounts are specified in the smallest unit of the applicable currency.
			// For USD amount must be in cents.

			$paymentAmount = mpapp()->bundles()->currencies()->getAmountInSmallestCurrencyUnits(
				$payment->getCurrency(),
				$payment->getAmount()
			);

			$money->setAmount( $paymentAmount );
			// This currency must be the same as in Location from settings!
			$money->setCurrency( $payment->getCurrency() );

			// Every payment you process with the SDK must have a unique idempotency key.
			// If you're unsure whether a particular payment succeeded, you can reattempt
			// it with the same idempotency key without worrying about double charging the buyer.
			// But we have button Back in appointment form after failed payment and in this case
			// we have to be able to process the same payment transaction again with another
			// payment data. So we are forced to use unique idempotency key for each payment attempt.
			$idempotencyKey = $payment->getId() . '-' . time();

			$createPaymentRequest = new \Square\Models\CreatePaymentRequest(
				$payment->getTransactionId(),
				$idempotencyKey
			);
			$createPaymentRequest->setAmountMoney( $money );
			$createPaymentRequest->setLocationId( $this->getOption( static::UNPREFIXED_SETTING_NAME_LOCATION_ID ) );

			// https://developer.squareup.com/docs/web-payments/sca
			if ( ! empty( $paymentData['verification_token'] ) ) {

				$createPaymentRequest->setVerificationToken( $paymentData['verification_token'] );
			}

			$response = $paymentsApi->createPayment( $createPaymentRequest );

			if ( $response->isSuccess() ) {

				mpa_payment_manager()->completePayment(
					$payment,
					sprintf(
						// Translators: %s is a Payment transaction id.
						esc_html__( 'Payment for a Square payment token %s succeeded.', 'mpa-square-payments' ),
						$payment->getTransactionId()
					)
				);
			} else {

				$errorMessage = sprintf(
					// Translators: %s is a Payment transaction id.
					esc_html__( 'Failed to process payment for a Square payment token %s.', 'mpa-square-payments' ),
					$payment->getTransactionId()
				);

				if ( ! empty( $response->getErrors() ) ) {

					foreach ( $response->getErrors() as $error ) {

						if ( ! empty( $error->getDetail() ) ) {

							$errorMessage .= ' ' . $error->getDetail();
						}
					}
				}

				throw new \Exception( $errorMessage );
			}
		} catch ( \Throwable $e ) {

			$payment->addLog(
				__( 'An error occurred during the payment process.', 'mpa-square-payments' ) . ' ' . $e->getMessage()
			);

			throw new \Exception(
				__( 'An error occurred during the payment process.', 'mpa-square-payments' ) . ' ' . $e->getMessage()
			);
		}
	}
}
