class SquarePaymentsGateway {
	/**
	 * @param {Object} $mountWrapper
	 * @param {Cart} cart
	 */
	constructor($billingFields, cart) {
		/**
		 * @var {String}
		 */
		this.gatewayId = 'square';

		/**
		 * @var {Object}
		 */
		this.settings = {};

		/**
		 * @var {Object}
		 */
		this.$billingFields = $billingFields;

		/**
		 * @var {Promise}
		 */
		this.loadPromise = null;

		/**
		 * @var {Boolean}
		 */
		this.isEnabled = false;

		/**
		 * @var {Boolean}
		 */
		this.isMounted = false;

		/**
		 * @var {Cart}
		 */
		this.cart = cart;

		this.$gatewayPreloader = $billingFields.parent().find('.mpa-payment-gateway-title .mpa-preloader');

		/**
		 * @var {String}
		 */
		this.processingPaymentMethod = null;

		/**
		 * @var {Object}
		 */
		this.paymentMethods = {};

		/**
		 * @var {Square.payments}
		 */
		this.squarePayments = null;
	}

	/**
	 * @return {Promise}
	 */
	load() {

		this.loadPromise = wp.apiRequest({
			path: '/motopress/appointment/v1/payments/settings',
			type: 'GET',
			data: {
				gateway_id: this.gatewayId,
			}
		})
		.catch(error => console.error(error.message) || {})
		.then(settings => {
			jQuery.extend(this.settings, settings);
			return this;
		});

		return this.loadPromise;
	}

	/**
	 * @return {Promise}
	 */
	ready() {
		return this.loadPromise;
	}

	/**
	 * @access protected
	 * @param {Object} $billingFields
	 */
	mount($billingFields) {

		this.$gatewayPreloader.removeClass('mpa-hide');

		// we need to hide with visibility because otherwise card iframe has wrong height
		this.$billingFields.css('visibility', 'hidden');

		this.$billingFields.find('input[name="mpa-square-payments__method-switcher"]').change((e) => {

			this.$billingFields.find('.mpa-square-payments__method').addClass('mpa-hide');

			if ('card' === e.target.value) {

				this.processingPaymentMethod = 'card';
				this.$billingFields.find('.mpa-square-payments__card').removeClass('mpa-hide');

			} else if ('gift-card' === e.target.value) {

				this.processingPaymentMethod = 'gift-card';
				this.$billingFields.find('.mpa-square-payments__gift-card').removeClass('mpa-hide');
			}
		});

		this.ready().then(async () => {

			this.squarePayments = Square.payments(this.settings.application_id, this.settings.location_id);


			if (this.settings.enabled_payment_methods.includes('card')) {

				const card = await this.squarePayments.card();
				
				this.paymentMethods['card'] = card;

				await card.attach($billingFields.find('.mpa-square-payments__card')[0]);
			}

			if (this.settings.enabled_payment_methods.includes('gift-card')) {

				const giftCard = await this.squarePayments.giftCard();

				this.paymentMethods['gift-card'] = giftCard;

				await giftCard.attach($billingFields.find('.mpa-square-payments__gift-card')[0]);
			}

			this.processingPaymentMethod = this.settings.enabled_payment_methods[0];

			this.$gatewayPreloader.addClass('mpa-hide');
			this.$billingFields.css('visibility', 'visible');
		});
	}

	enable() {

		if (!this.isEnabled) {

			if (!this.isMounted) {

				this.mount(this.$billingFields);
				this.isMounted = true;
			}

			this.$billingFields.removeClass('mpa-hide');
			this.isEnabled = true;
		}
	}

	disable() {

		if (this.isEnabled) {

			this.$billingFields.addClass('mpa-hide');
			this.isEnabled = false;
		}
	}

	async reset() {

		if (this.paymentMethods['card']) {
			await this.paymentMethods['card'].clear();
		}
		if (this.paymentMethods['gift-card']) {
			await this.paymentMethods['gift-card'].clear();
		}
		// after we clear fields Square SDK shows error for empty card field
		// so we need to clear it on reset
		// we can not use destroy and new mount because Square SDK adds
		// card field with extra empty line after carf form
		jQuery('.sq-card-wrapper').removeClass('sq-error');
		jQuery('.sq-card-message').removeClass('sq-visible');
	}

	onCartChange(cart) {

		// TODO: cart total price could changed so we need to reset payment methods
		// with payment request like Apple Pay, Google Pay and so on here
	}

	/**
	 * @return {Boolean}
	 */
	isValid() {
		// we can not check user input in iframes
		// we will know about errors after payment started in processPayment()
		return true;
	}

	/**
	 * @param {Cart} cart
	 * @param {Object} bookingDetails Booking and payment ID/UID.
	 * @return {Promise}
	 */
	processPayment(cart, bookingDetails) {

		this.$gatewayPreloader.removeClass('mpa-hide');

		let paymentDetails = jQuery.extend(
			cart.paymentDetails,
			{
				payment_method: this.processingPaymentMethod,
			}
		);

		return this.paymentMethods[this.processingPaymentMethod].tokenize()
			.catch(error => {

				console.error(error.message);

				this.$gatewayPreloader.addClass('mpa-hide');
			})
			.then(async result => {

				if (result && 'OK' === result.status) {

					if ('card' === this.processingPaymentMethod) {

						paymentDetails.payment_method += ': ' + result.details.card.brand + ' ' +
							result.details.card.last4;

						// https://developer.squareup.com/docs/web-payments/sca
						if (this.cart.customerDetails.email || this.cart.customerDetails.phone) {

							const cartOrder = this.cart.getOrder();

							const verificationDetails = {
								// we must send string values
								amount: '' + cartOrder.total,
								billingContact: {
									// givenName: 'John',
									// familyName: 'Doe',
									// addressLines: ['123 Main Street', 'Apartment 1'],
									// city: 'London',
									// state: 'LND',
									// countryCode: 'GB',
								},
								currencyCode: this.settings.currency_code,
								intent: 'CHARGE',
							};

							if (cartOrder.deposit) {
								verificationDetails.amount = '' + cartOrder.deposit;
							}

							if (this.cart.customerDetails.email) {
								verificationDetails.billingContact.email = this.cart.customerDetails.email;
							}

							if (this.cart.customerDetails.phone) {
								verificationDetails.billingContact.phone = this.cart.customerDetails.phone;
							}

							try {

								const verificationResults = await this.squarePayments.verifyBuyer(
									result.token,
									verificationDetails,
								);

								paymentDetails.verification_token = verificationResults.token;

							} catch (error) {

								this.$gatewayPreloader.addClass('mpa-hide');
								throw error;
							}
						}
					}

					paymentDetails.payment_token = result.token;

					await wp.apiRequest({
						path: '/motopress/appointment/v1/payments/prepare',
						type: 'POST',
						data: {
							payment_details: paymentDetails,
						}
					}),

					this.$gatewayPreloader.addClass('mpa-hide');

					return paymentDetails;

				} else {

					if (result) {

						let errorMessage = `Start payment failed with status: ${result.status}`;

						if (result.errors) {
							errorMessage += ` and errors: ${JSON.stringify(result.errors)}`;
						}

						console.error(errorMessage);
					}

					this.$gatewayPreloader.addClass('mpa-hide');

					throw new Error('Failed to start payment processing.');
				}
			});
	}
}

// registed gateway in appointment gateways
wp.hooks.addFilter( 
    'mpa_create_gateway',
    'MotoPress\Appointment\Addons\SquarePayments',
    (gateway, gatewayId, $billingFields, cart) => {

        if ( !gateway && 'square' === gatewayId ) {

            gateway = new SquarePaymentsGateway( $billingFields, cart );
        }

        return gateway;
    },
    10
);
