<?php

namespace MotoPress\Appointment\Addons\WooCommerce;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Allows plugins to use their own update API.
 *
 * @author Easy Digital Downloads
 * @version 1.6.12
 */
class EDDPluginUpdater {

	private $apiUrl     = '';
	private $apiData    = array();
	private $name       = '';
	private $slug       = '';
	private $version    = '';
	private $wpOverride = false;
	private $cacheKey   = '';
	private $beta       = false;

	/**
	 * Class constructor.
	 *
	 * @uses plugin_basename()
	 * @uses hook()
	 *
	 * @param string  $_apiUrl     The URL pointing to the custom API endpoint.
	 * @param string  $_pluginFile Path to the plugin file.
	 * @param array   $_apiData    Optional data to send with API calls.
	 */
	public function __construct( $_apiUrl, $_pluginFile, $_apiData = null ) {
		global $edd_plugin_data;

		$this->apiUrl     = trailingslashit( $_apiUrl );
		$this->apiData    = $_apiData;
		$this->name       = plugin_basename( $_pluginFile );
		$this->slug       = basename( $_pluginFile, '.php' );
		$this->version    = $_apiData['version'];
		$this->wpOverride = isset( $_apiData['wp_override'] ) ? (bool) $_apiData['wp_override'] : false;
		$this->beta       = ! empty( $this->apiData['beta'] ) ? true : false;
		$this->cacheKey   = md5( serialize( $this->slug . $this->apiData['license'] . $this->beta ) );

		$edd_plugin_data[ $this->slug ] = $this->apiData;

		// Set up hooks.
		$this->init();
	}

	/**
	 * Set up WordPress filters to hook into WP's update process.
	 *
	 * @uses add_filter()
	 *
	 * @return void
	 */
	public function init() {
		add_filter( 'pre_set_site_transient_update_plugins', array( $this, 'checkUpdate' ) );
		add_filter( 'plugins_api', array( $this, 'pluginsApiFilter' ), 10, 3 );
		remove_action( 'after_plugin_row_' . $this->name, 'wp_plugin_update_row', 10 );
		add_action( 'after_plugin_row_' . $this->name, array( $this, 'showUpdateNotification' ), 10, 2 );
		add_action( 'admin_init', array( $this, 'showChangelog' ) );
	}

	/**
	 * Check for Updates at the defined API endpoint and modify the update array.
	 *
	 * This function dives into the update API just when WordPress creates its update array,
	 * then adds a custom API call and injects the custom plugin data retrieved from the API.
	 * It is reassembled from parts of the native WordPress plugin update code.
	 * See wp-includes/update.php line 121 for the original wp_update_plugins() function.
	 *
	 * @uses apiRequest()
	 *
	 * @param array  $_transientData Update array build by WordPress.
	 * @return array Modified update array with custom plugin data.
	 */
	public function checkUpdate( $_transientData ) {

		global $pagenow;

		if ( ! is_object( $_transientData ) ) {

			$_transientData = new \stdClass();
		}

		if ( 'plugins.php' == $pagenow && is_multisite() ) {

			return $_transientData;
		}

		if ( ! empty( $_transientData->response ) && ! empty( $_transientData->response[ $this->name ] ) && false === $this->wpOverride ) {

			return $_transientData;
		}

		$versionInfo = $this->getCachedVersionInfo();

		if ( false === $versionInfo ) {

			$versionInfo = $this->apiRequest(
				'plugin_latest_version',
				array(
					'slug' => $this->slug,
					'beta' => $this->beta,
				)
			);

			$this->setVersionInfoCache( $versionInfo );
		}

		if ( false !== $versionInfo && is_object( $versionInfo ) && isset( $versionInfo->new_version ) ) {

			if ( version_compare( $this->version, $versionInfo->new_version, '<' ) ) {

				$_transientData->response[ $this->name ] = $versionInfo;

			}

			$_transientData->last_checked           = current_time( 'timestamp' );
			$_transientData->checked[ $this->name ] = $this->version;
		}

		return $_transientData;
	}

	/**
	 * Show update notification row -- needed for multisite subsites, because WP won't tell you otherwise!
	 *
	 * @param string  $file
	 * @param array   $plugin
	 */
	public function showUpdateNotification( $file, $plugin ) {

		if ( is_network_admin() ) {
			return;
		}

		if ( ! current_user_can( 'update_plugins' ) ) {
			return;
		}

		if ( ! is_multisite() ) {
			return;
		}

		if ( $this->name != $file ) {
			return;
		}

		// Remove our filter on the site transient
		remove_filter( 'pre_set_site_transient_update_plugins', array( $this, 'checkUpdate' ), 10 );

		$updateCache = get_site_transient( 'update_plugins' );

		$updateCache = is_object( $updateCache ) ? $updateCache : new \stdClass();

		if ( empty( $updateCache->response ) || empty( $updateCache->response[ $this->name ] ) ) {

			$versionInfo = $this->getCachedVersionInfo();

			if ( false === $versionInfo ) {

				$versionInfo = $this->apiRequest(
					'plugin_latest_version',
					array(
						'slug' => $this->slug,
						'beta' => $this->beta,
					)
				);

				$this->setVersionInfoCache( $versionInfo );
			}

			if ( ! is_object( $versionInfo ) ) {
				return;
			}

			if ( version_compare( $this->version, $versionInfo->new_version, '<' ) ) {

				$updateCache->response[ $this->name ] = $versionInfo;
			}

			$updateCache->last_checked           = current_time( 'timestamp' );
			$updateCache->checked[ $this->name ] = $this->version;

			set_site_transient( 'update_plugins', $updateCache );

		} else {

			$versionInfo = $updateCache->response[ $this->name ];
		}

		// Restore our filter
		add_filter( 'pre_set_site_transient_update_plugins', array( $this, 'checkUpdate' ) );

		if ( ! empty( $updateCache->response[ $this->name ] ) && version_compare( $this->version, $versionInfo->new_version, '<' ) ) {

			// build a plugin list row, with update notification
			$wp_list_table = _get_list_table( 'WP_Plugins_List_Table' );
			# <tr class="plugin-update-tr"><td colspan="' . $wp_list_table->get_column_count() . '" class="plugin-update colspanchange">
			echo '<tr class="plugin-update-tr" id="' . esc_attr( $this->slug ) . '-update" data-slug="' .
			esc_attr( $this->slug ) . '" data-plugin="' . esc_attr( $this->slug ) . '/' . esc_attr( $file ) . '">';
			echo '<td colspan="3" class="plugin-update colspanchange">';
			echo '<div class="update-message notice inline notice-warning notice-alt">';

			$changelogLink = self_admin_url( 'index.php?edd_sl_action=view_plugin_changelog&plugin=' . $this->name . '&slug=' . $this->slug . '&TB_iframe=true&width=772&height=911' );

			if ( empty( $versionInfo->download_link ) ) {

				printf(
					// translators: %1$s is plugin name, %2$s is start part of the link html code to changelog, %3$s is closed link tag.
					esc_html__( 'There is a new version of %1$s available. %2$sView version %3$s details%4$s.', 'easy-digital-downloads' ),
					esc_html( $versionInfo->name ),
					'<a target="_blank" class="thickbox" href="' . esc_url( $changelogLink ) . '">',
					esc_html( $versionInfo->new_version ),
					'</a>'
				);
			} else {

				printf(
					// translators: %1$s is plugin name, %2$s is start part of the link html code to changelog, %3$s is number of version, %4$s is closed link tag, %5$s is start of the link to upgrade plugin, %6$s is clodes link tag.
					esc_html__( 'There is a new version of %1$s available. %2$sView version %3$s details%4$s or %5$supdate now%6$s.', 'easy-digital-downloads' ),
					esc_html( $versionInfo->name ),
					'<a target="_blank" class="thickbox" href="' . esc_url( $changelogLink ) . '">',
					esc_html( $versionInfo->new_version ),
					'</a>',
					'<a href="' . esc_url( wp_nonce_url( self_admin_url( 'update.php?action=upgrade-plugin&plugin=' ) . $this->name, 'upgrade-plugin_' . $this->name ) ) . '">',
					'</a>'
				);
			}

			do_action( "in_plugin_update_message-{$file}", $plugin, $versionInfo );

			echo '</div></td></tr>';
		}
	}

	/**
	 * Updates information on the "View version x.x details" page with custom data.
	 *
	 * @uses apiRequest()
	 *
	 * @param mixed   $_data
	 * @param string  $_action
	 * @param object  $_args
	 * @return object $_data
	 */
	public function pluginsApiFilter( $_data, $_action = '', $_args = null ) {

		if ( 'plugin_information' != $_action ) {
			return $_data;
		}

		if ( ! isset( $_args->slug ) || ( $_args->slug != $this->slug ) ) {
			return $_data;
		}

		$toSend = array(
			'slug'   => $this->slug,
			'is_ssl' => is_ssl(),
			'fields' => array(
				'banners' => array(),
				'reviews' => false,
			),
		);

		$cacheKey = 'edd_api_request_' . md5( serialize( $this->slug . $this->apiData['license'] . $this->beta ) );

		// Get the transient where we store the api request for this plugin for 24 hours
		$eddApiRequestTransient = $this->getCachedVersionInfo( $cacheKey );

		//If we have no transient-saved value, run the API, set a fresh transient with the API value, and return that value too right now.
		if ( empty( $eddApiRequestTransient ) ) {

			$apiResponse = $this->apiRequest( 'plugin_information', $toSend );

			// Expires in 3 hours
			$this->setVersionInfoCache( $apiResponse, $cacheKey );

			if ( false !== $apiResponse ) {
				$_data = $apiResponse;
			}
		} else {
			$_data = $eddApiRequestTransient;
		}

		// Convert sections into an associative array, since we're getting an object, but Core expects an array.
		if ( isset( $_data->sections ) && ! is_array( $_data->sections ) ) {

			$new_sections = array();

			foreach ( $_data->sections as $key => $value ) {
				$new_sections[ $key ] = $value;
			}

			$_data->sections = $new_sections;
		}

		// Convert banners into an associative array, since we're getting an object, but Core expects an array.
		if ( isset( $_data->banners ) && ! is_array( $_data->banners ) ) {

			$new_banners = array();

			foreach ( $_data->banners as $key => $value ) {
				$new_banners[ $key ] = $value;
			}

			$_data->banners = $new_banners;
		}

		return $_data;
	}

	/**
	 * Disable SSL verification in order to prevent download update failures
	 *
	 * @param array   $args
	 * @param string  $url
	 * @return object $array
	 */
	public function http_request_args( $args, $url ) {
		// If it is an https request and we are performing a package download, disable ssl verification
		if ( strpos( $url, 'https://' ) !== false && strpos( $url, 'edd_action=package_download' ) ) {
			$args['sslverify'] = false;
		}
		return $args;
	}

	/**
	 * Calls the API and, if successful, returns the object delivered by the API.
	 *
	 * @uses get_bloginfo()
	 * @uses wp_remote_post()
	 * @uses is_wp_error()
	 *
	 * @param string  $_action The requested action.
	 * @param array   $_data   Parameters for the API action.
	 * @return false|object
	 */
	private function apiRequest( $_action, $_data ) {

		global $wp_version;

		$data = array_merge( $this->apiData, $_data );

		if ( $data['slug'] != $this->slug ) {
			return;
		}

		if ( trailingslashit( home_url() ) == $this->apiUrl ) {
			return false; // Don't allow a plugin to ping itself
		}

		$apiParams = array(
			'edd_action' => 'get_version',
			'license'    => ! empty( $data['license'] ) ? $data['license'] : '',
			'item_name'  => isset( $data['item_name'] ) ? $data['item_name'] : false,
			'item_id'    => isset( $data['item_id'] ) ? $data['item_id'] : false,
			'version'    => isset( $data['version'] ) ? $data['version'] : false,
			'slug'       => $data['slug'],
			'author'     => $data['author'],
			'url'        => home_url(),
			'beta'       => ! empty( $data['beta'] ),
		);

		$request = wp_remote_post(
			$this->apiUrl,
			array(
				'timeout'   => 15,
				'sslverify' => false,
				'body'      => $apiParams,
			)
		);

		if ( ! is_wp_error( $request ) ) {

			$request = json_decode( wp_remote_retrieve_body( $request ) );
		}

		if ( $request && isset( $request->sections ) ) {

			$request->sections = maybe_unserialize( $request->sections );

		} else {

			$request = false;
		}

		if ( $request && isset( $request->banners ) ) {

			$request->banners = maybe_unserialize( $request->banners );
		}

		if ( ! empty( $request->sections ) ) {

			foreach ( $request->sections as $key => $section ) {
				$request->$key = (array) $section;
			}
		}

		return $request;
	}

	public function showChangelog() {
		global $edd_plugin_data;

		if ( empty( $_REQUEST['edd_sl_action'] ) || 'view_plugin_changelog' != $_REQUEST['edd_sl_action'] ) {
			return;
		}

		if ( empty( $_REQUEST['plugin'] ) ) {
			return;
		}

		if ( empty( $_REQUEST['slug'] ) ) {
			return;
		}

		if ( ! current_user_can( 'update_plugins' ) ) {

			wp_die(
				esc_html__( 'You do not have permission to install plugin updates', 'easy-digital-downloads' ),
				esc_html__( 'Error', 'easy-digital-downloads' ),
				array( 'response' => 403 )
			);
		}

		$data        = $edd_plugin_data[ sanitize_text_field( wp_unslash( $_REQUEST['slug'] ) ) ];
		$beta        = ! empty( $data['beta'] ) ? true : false;
		$cacheKey    = md5( 'edd_plugin_' . sanitize_key( $_REQUEST['plugin'] ) . '_' . $beta . '_version_info' );
		$versionInfo = $this->getCachedVersionInfo( $cacheKey );

		if ( false === $versionInfo ) {

			$apiParams = array(
				'edd_action' => 'get_version',
				'item_name'  => isset( $data['item_name'] ) ? $data['item_name'] : false,
				'item_id'    => isset( $data['item_id'] ) ? $data['item_id'] : false,
				'slug'       => sanitize_text_field( wp_unslash( $_REQUEST['slug'] ) ),
				'author'     => $data['author'],
				'url'        => home_url(),
				'beta'       => ! empty( $data['beta'] ),
			);

			$request = wp_remote_post(
				$this->apiUrl,
				array(
					'timeout'   => 15,
					'sslverify' => false,
					'body'      => $apiParams,
				)
			);

			if ( ! is_wp_error( $request ) ) {

				$versionInfo = json_decode( wp_remote_retrieve_body( $request ) );
			}

			if ( ! empty( $versionInfo ) && isset( $versionInfo->sections ) ) {

				$versionInfo->sections = maybe_unserialize( $versionInfo->sections );

			} else {

				$versionInfo = false;
			}

			if ( ! empty( $versionInfo ) ) {

				foreach ( $versionInfo->sections as $key => $section ) {
					$versionInfo->$key = (array) $section;
				}
			}

			$this->setVersionInfoCache( $versionInfo, $cacheKey );
		}

		if ( ! empty( $versionInfo ) && isset( $versionInfo->sections['changelog'] ) ) {

			echo '<div style="background:#fff;padding:10px;">' . wp_kses_post( $versionInfo->sections['changelog'] ) . '</div>';
		}

		exit;
	}

	public function getCachedVersionInfo( $cacheKey = '' ) {
		// uncomment this line for testing
		//return false;

		if ( empty( $cacheKey ) ) {
			$cacheKey = $this->cacheKey;
		}

		$cache = get_option( $cacheKey );

		if ( empty( $cache['timeout'] ) || current_time( 'timestamp' ) > $cache['timeout'] ) {
			return false; // Cache is expired
		}

		return json_decode( $cache['value'] );
	}

	public function setVersionInfoCache( $value = '', $cacheKey = '' ) {
		if ( empty( $cacheKey ) ) {
			$cacheKey = $this->cacheKey;
		}

		$data = array(
			'timeout' => strtotime( '+3 hours', current_time( 'timestamp' ) ),
			'value'   => json_encode( $value ),
		);

		update_option( $cacheKey, $data );
	}
}
