<?php

namespace MotoPress\Appointment\Addons\WooCommerce;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}


class PluginLifecycleHandler {

	private const REQUIRED_APPOINTMENT_PLUGIN_VERSION = '1.18.1';
	private const REQUIRED_WOOCOMMERCE_PLUGIN_VERSION = '6.4.0';

	private $isWPEnvironmentSuitedForPlugin = true;

	public function __construct( string $pluginMainFilePath ) {

		$this->checkPluginDependencies();

		register_activation_hook(
			$pluginMainFilePath,
			function() {
				$this->activatePlugin();
			}
		);

		register_deactivation_hook(
			$pluginMainFilePath,
			function() {
				$this->deactivatePlugin();
			}
		);

		register_uninstall_hook( $pluginMainFilePath, array( __CLASS__, 'uninstallPlugin' ) );

		// initialize EDD updater
		if ( ! wp_doing_ajax() ) {

			add_action(
				'admin_init',
				function() use ( $pluginMainFilePath ) {
					if ( MPAWooCommerce::isEDDLicenseEnabled() ) {

						new EDDPluginUpdater(
							MPAWooCommerce::getPluginSourceServerUrl(),
							$pluginMainFilePath,
							array(
								'version' => MPAWooCommerce::getPluginVersion(),
								'license' => MPAWooCommerce::getLicenseKey(),
								'item_id' => MPAWooCommerce::getProductId(),
								'author'  => MPAWooCommerce::getPluginAuthorName(),
							)
						);
					}
				}
			);
		}
	}

	private function checkPluginDependencies(): void {

		$isRequiredVersionOfAppointmentPluginActive = false;

		if ( defined( 'MotoPress\Appointment\VERSION' ) ) {

			$isRequiredVersionOfAppointmentPluginActive = version_compare(
				\MotoPress\Appointment\VERSION,
				static::REQUIRED_APPOINTMENT_PLUGIN_VERSION,
				'>='
			);
		}

		if ( ! $isRequiredVersionOfAppointmentPluginActive ) {

			$this->isWPEnvironmentSuitedForPlugin = false;

			BackendHelper::addErrorAdminNotice(
				sprintf(
					// translators: %1$s is the name of the required plugin, %2$s is the required plugin version like 1.x.x
					__( 'The Appointment Booking WooCommerce Payments addon requires the %1$s plugin %2$s version or higher. Install and activate it for proper work.', 'mpa-woocommerce' ),
					'Appointment Booking',
					static::REQUIRED_APPOINTMENT_PLUGIN_VERSION
				)
			);
		}

		$isRequiredVersionOfWooCommercePluginActive = false;

		if ( function_exists( 'wc' ) ) {

			$woocommerce = wc();

			$isRequiredVersionOfWooCommercePluginActive = $woocommerce &&
				property_exists( $woocommerce, 'version' ) &&
				version_compare(
					$woocommerce->version,
					self::REQUIRED_WOOCOMMERCE_PLUGIN_VERSION,
					'>='
				);
		}

		if ( ! $isRequiredVersionOfWooCommercePluginActive ) {

			$this->isWPEnvironmentSuitedForPlugin = false;

			BackendHelper::addErrorAdminNotice(
				sprintf(
					// translators: %1$s is the name of the required plugin, %2$s is the required plugin version like 1.x.x
					__( 'The Appointment Booking WooCommerce Payments addon requires the %1$s plugin %2$s version or higher. Install and activate it for proper work.', 'mpa-woocommerce' ),
					'WooCommerce',
					static::REQUIRED_WOOCOMMERCE_PLUGIN_VERSION
				)
			);
		}
	}

	public function isWPEnvironmentSuitedForPlugin(): bool {
		return $this->isWPEnvironmentSuitedForPlugin;
	}

	private function activatePlugin() {}

	private function deactivatePlugin() {}

	// execute this method from uninstall.php when it will be needed
	public static function uninstallPlugin() {}
}
