<?php

namespace MotoPress\Appointment\Addons\WooCommerce;

use MotoPress\Appointment\Entities\Booking;
use MotoPress\Appointment\Entities\Payment;
use MotoPress\Appointment\Entities\Reservation;
use MotoPress\Appointment\Payments\Gateways\AbstractPaymentGateway;
use MotoPress\Appointment\PostTypes\Statuses\PaymentStatuses;
use MotoPress\Appointment\PostTypes\Statuses\BookingStatuses;
use MotoPress\Appointment\Services\BookingService;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}


class WooCommercePaymentsGateway extends AbstractPaymentGateway {

	const WOO_ORDER_ITEM_META_KEY_MPA_PAYMENT_ID  = 'mpa_payment_id';
	const WOO_ORDER_ITEM_META_KEY_MPA_COUPON_CODE = 'mpa_coupon_code';
	const WOO_ORDER_ITEM_META_KEY_MPA_DEPOSIT     = 'mpa_deposit';

	public function __construct() {

		parent::__construct();

		// register plugins custom settings field
		add_filter(
			'mpa_mpawoo-license-settings_field_class',
			function() {
				return 'MotoPress\Appointment\Addons\WooCommerce\PluginLicenseSettingsField';
			}
		);

		PluginLicenseSettingsField::registerField();

		add_action(
			'mpa_settings_saved',
			function( $currentTab, $currentSection ) {
				// reread options after save
				$this->setupProperties();

				if ( 'payment' === $currentTab && $currentSection === $this->getId() ) {

					$this->setFakeDefaultWooCommerceProductPrice();
				}
			},
			10,
			2
		);

		// we need to make sure that all setting already stored
		add_action(
			'in_admin_header',
			function() {
				$this->checkSettingsAndShowAdminWarnings();
			}
		);

		if ( $this->isActive() ) {
			$this->customizeWooCommerceCartAndOrderData();
			$this->customizeWooCommercePages();
			$this->customizeWooCommercePaymentProcessing();
		}
	}

	public function getId(): string {
		return 'woocommerce';
	}

	public function getName(): string {
		return __( 'WooCommerce', 'mpa-woocommerce' );
	}

	protected function getDefaultPublicName(): string {
		return __( 'Pay by WooCommerce', 'mpa-woocommerce' );
	}

	public function isSupportsSandbox(): bool {
		return false;
	}

	/**
	 * Set default fake price to avoid failing is_purchasable
	 */
	private function setFakeDefaultWooCommerceProductPrice() {

		$wooCommerceProductId = MPAWooCommerce::getWooCommerceProductId();

		if ( 0 < $wooCommerceProductId ) {

			$product = wc_get_product( $wooCommerceProductId );

			if ( ! $product ) {
				return;
			}

			if ( 1 != $product->get_price() ) {

				$product->set_regular_price( 1 );
				$product->save();
			}
		}
	}

	public function getFields() {

		$fields = parent::getFields();

		$wooProductsList = array( 0 => __( 'Select product', 'mpa-woocommerce' ) . '...' );

		$wooProducts = get_posts(
			array(
				'numberposts'      => -1,
				'post_type'        => 'product',
				'suppress_filters' => true,
			)
		);

		foreach ( $wooProducts as $product ) {
			$wooProductsList[ $product->ID ] = $product->post_title;
		}

		$fields[ $this->getOptionNameRaw( 'woo_product_id' ) ] = array(
			'type'        => 'select',
			'label'       => __( 'WooCommerce Product', 'mpa-woocommerce' ),
			'description' => sprintf(
				// translators: %s is new woocommerce product link
				__( 'Go to <a href="%s">WooCommerce > Products</a> and create a product that will be used for appointments.', 'mpa-woocommerce' ),
				get_admin_url( null, 'post-new.php?post_type=product' )
			),
			'size'        => 'regular',
			'options'     => $wooProductsList,
			'default'     => 0,
		);

		if ( MPAWooCommerce::isEDDLicenseEnabled() ) {

			$fields[ MPAWooCommerce::SETTINGS_NAME_PLUGIN_LICENSE_KEY ] = array(
				'type'        => PluginLicenseSettingsField::TYPE,
				'label'       => __( 'License Key', 'mpa-woocommerce' ),
				'description' => __( "The License Key is required in order to get automatic plugin updates and support. You can manage your License Key in your personal account. <a href='https://motopress.zendesk.com/hc/en-us/articles/202812996-How-to-use-your-personal-MotoPress-account' target='blank'>Learn more</a>.", 'mpa-woocommerce' ),
				'size'        => 'regular',
			);
		}

		return $fields;
	}

	/**
	 * @see \MotoPress\Appointment\Shortcodes\AppointmentFormShortcode::enqueueScripts()
	 */
	public function enqueueScripts() {

		if ( ! is_admin() ) {

			wp_enqueue_script(
				'mpawoo-public',
				trailingslashit( MPAWooCommerce::getPluginUrl() ) . 'js/public.min.js',
				array( 'mpa-public' ),
				'1.1',
				true
			);
			wp_localize_script(
				'mpawoo-public',
				'mpawooScriptData',
				array(
					'wooCheckoutUrl' => wc_get_checkout_url(),
					'wooProductId'   => MPAWooCommerce::getLocalizedWooCommerceProductId(),
				)
			);
		}
	}

	private function checkSettingsAndShowAdminWarnings() {

		if ( ! $this->isEnabled() || ! is_admin() ) {
			return;
		}

		if ( 0 >= MPAWooCommerce::getWooCommerceProductId() ) {

			BackendHelper::addErrorAdminNotice(
				__( 'To make the WooCommerce Payments plugin work, add a WooCommerce Product!', 'mpa-woocommerce' )
			);
		}

		if ( get_woocommerce_currency() !== mpapp()->settings()->getCurrency() ) {

			BackendHelper::addErrorAdminNotice(
				__( 'To make the WooCommerce Payments plugin work, set up the same currency for it as in WooCommerce!', 'mpa-woocommerce' )
			);
		}
	}

	/**
	 * Method for backwards compatibility with appointment before 1.18.0
	 * TODO: remove after REQUIRED_APPOINTMENT_PLUGIN_VERSION will be 1.18.0 or later
	 */
	private function getCustomerName( Booking $booking ): string {

		if ( ! method_exists( $booking, 'getCustomerName' ) ) {
			return $booking->customer->name;
		}

		return $booking->getCustomerName();
	}

	/**
	 * Method for backwards compatibility with appointment before 1.18.0
	 * TODO: remove after REQUIRED_APPOINTMENT_PLUGIN_VERSION will be 1.18.0 or later
	 */
	private function getCustomerPhone( Booking $booking ): string {

		if ( ! method_exists( $booking, 'getCustomerPhone' ) ) {
			return $booking->customer->phone;
		}

		return $booking->getCustomerPhone();
	}

	/**
	 * Method for backwards compatibility with appointment before 1.18.0
	 * TODO: remove after REQUIRED_APPOINTMENT_PLUGIN_VERSION will be 1.18.0 or later
	 */
	private function getCustomerEmail( Booking $booking ): string {

		if ( ! method_exists( $booking, 'getCustomerEmail' ) ) {
			return $booking->customer->email;
		}

		return $booking->getCustomerEmail();
	}

	/**
	 * Method for backwards compatibility with appointment before 1.18.0
	 * TODO: remove after REQUIRED_APPOINTMENT_PLUGIN_VERSION will be 1.18.0 or later
	 */
	private function getPaymentStatus( Payment $payment ):string {

		if ( ! method_exists( $payment, 'getStatus' ) ) {
			return $payment->status;
		}

		return $payment->getStatus();
	}

	/**
	 * Method for backwards compatibility with appointment before 1.18.0
	 * TODO: remove after REQUIRED_APPOINTMENT_PLUGIN_VERSION will be 1.18.0 or later
	 */
	private function getEventStart( Reservation $reservation ): \DateTime {

		if ( ! method_exists( $reservation, 'getServiceTime' ) ) {
			return $reservation->serviceTime->startTime->setTimezone( wp_timezone() );
		}

		return $reservation->getServiceTime()->startTime->setTimezone( wp_timezone() );
	}

	/**
	 * Method for backwards compatibility with appointment before 1.18.0
	 * TODO: remove after REQUIRED_APPOINTMENT_PLUGIN_VERSION will be 1.18.0 or later
	 */
	private function getServiceId( Reservation $reservation ): int {

		if ( ! method_exists( $reservation, 'getServiceId' ) ) {
			return $reservation->serviceId;
		}

		return $reservation->getServiceId();
	}

	/**
	 * Method for backwards compatibility with appointment before 1.18.0
	 * TODO: remove after REQUIRED_APPOINTMENT_PLUGIN_VERSION will be 1.18.0 or later
	 */
	private function getPaymentCurrency( Payment $payment ): string {

		if ( ! method_exists( $payment, 'getCurrency' ) ) {
			return $payment->currency;
		}

		return $payment->getCurrency();
	}

	/**
	 * Method for backwards compatibility with appointment before 1.18.0
	 * TODO: remove after REQUIRED_APPOINTMENT_PLUGIN_VERSION will be 1.18.0 or later
	 */
	private function getPaymentBookingId( Payment $payment ): int {

		if ( ! method_exists( $payment, 'getBookingId' ) ) {
			return $payment->bookingId;
		}

		return $payment->getBookingId();
	}

	/**
	 * Method for backwards compatibility with appointment before 1.18.0
	 * TODO: remove after REQUIRED_APPOINTMENT_PLUGIN_VERSION will be 1.18.0 or later
	 */
	private function setCustomerName( Booking $booking, string $name ) {

		if ( ! method_exists( $booking, 'setCustomerName' ) ) {
			$booking->customer->name = $name;
			return;
		}

		$booking->setCustomerName( $name );
	}

	/**
	 * Method for backwards compatibility with appointment before 1.18.0
	 * TODO: remove after REQUIRED_APPOINTMENT_PLUGIN_VERSION will be 1.18.0 or later
	 * @throws \Exception when email is invalid
	 */
	private function setCustomerEmail( Booking $booking, string $email ) {

		if ( ! method_exists( $booking, 'setCustomerEmail' ) ) {
			$booking->customer->name = $email;
			return;
		}

		$booking->setCustomerEmail( $email );
	}

	/**
	 * Method for backwards compatibility with appointment before 1.18.0
	 * TODO: remove after REQUIRED_APPOINTMENT_PLUGIN_VERSION will be 1.18.0 or later
	 */
	private function setCustomerPhone( Booking $booking, string $phone ) {

		if ( ! method_exists( $booking, 'setCustomerPhone' ) ) {
			$booking->customer->name = $phone;
			return;
		}

		$booking->setCustomerPhone( $phone );
	}

	/**
	 * Method for backwards compatibility with appointment before 1.18.0
	 * TODO: remove after REQUIRED_APPOINTMENT_PLUGIN_VERSION will be 1.18.0 or later
	 */
	private function setStatus( Booking $booking, string $status ) {

		if ( ! method_exists( $booking, 'setStatus' ) ) {
			$booking->status = $status;
			return;
		}

		$booking->setStatus( $status );
	}

	/**
	 * Method for backwards compatibility with appointment before 1.18.0
	 * TODO: remove after REQUIRED_APPOINTMENT_PLUGIN_VERSION will be 1.18.0 or later
	 */
	private function getPaymentMethod( Payment $payment ): string {

		if ( ! method_exists( $payment, 'getPaymentMethod' ) ) {
			return $payment->paymentMethod;
		}

		return $payment->getPaymentMethod();
	}

	private function customizeWooCommerceCartAndOrderData() {

		// Make sure that appointment WC product will be alone in the cart
		add_filter(
			'woocommerce_add_to_cart_product_id',
			function( $wooProductId ) {

				if ( MPAWooCommerce::isWooCommerceProductForAppointmentPayments( $wooProductId ) ) {

					WC()->cart->empty_cart();

				} else {

					foreach ( WC()->cart->get_cart() as $cartItemData ) {

						if ( MPAWooCommerce::isWooCommerceProductForAppointmentPayments( $cartItemData['product_id'] ) ) {

							WC()->cart->empty_cart();
						}
					}
				}
				return $wooProductId;
			}
		);

		// Empty cart after order cancellation by user
		add_action(
			'woocommerce_cancelled_order',
			function( $wcOrderId ) {
				WC()->cart->empty_cart();
			},
			10,
			1
		);

		// add payment transaction id to the cart item product data
		add_action(
			'woocommerce_add_cart_item_data',
			function( $cartItemData, $wooProductId ) {
				// phpcs:ignore
				if ( ! isset( $_POST['mpa_booking_id'] ) ||
					! MPAWooCommerce::isWooCommerceProductForAppointmentPayments( $wooProductId ) ) {

					return $cartItemData;
				}
				// phpcs:ignore
				$booking = mpapp()->repositories()->booking()->findById( absint( wp_unslash( $_POST['mpa_booking_id'] ) ) );

				if ( ! $booking ) {
					return $cartItemData;
				}

				$paymentId = $booking->getExpectingPaymentId();

				if ( ! $paymentId ) {
					return $cartItemData;
				}

				$cartItemData[ static::WOO_ORDER_ITEM_META_KEY_MPA_PAYMENT_ID ] = $paymentId;
				// phpcs:ignore
				$cartItemData[ static::WOO_ORDER_ITEM_META_KEY_MPA_COUPON_CODE ] = isset( $_POST['mpa_coupon_code'] ) ? sanitize_text_field( wp_unslash( $_POST['mpa_coupon_code'] ) ) : '';
				// phpcs:ignore
				$cartItemData[ static::WOO_ORDER_ITEM_META_KEY_MPA_DEPOSIT ] = isset( $_POST['mpa_deposit'] ) && 'true' === $_POST['mpa_deposit'];

				return $cartItemData;
			},
			10,
			2
		);

		// Remove product with not existed or abandoned payments from the cart
		add_action(
			'wp_loaded',
			function() {
				if ( ! WC()->cart ) {
					return;
				}

				foreach ( WC()->cart->get_cart() as $cartItemKey => $cartItem ) {

					if ( MPAWooCommerce::isWooCommerceProductForAppointmentPayments( $cartItem['product_id'] ) ) {

						if ( ! array_key_exists( static::WOO_ORDER_ITEM_META_KEY_MPA_PAYMENT_ID, $cartItem ) ) {

							WC()->cart->remove_cart_item( $cartItemKey );
							continue;
						}

						$mpaPayment = mpapp()->repositories()->payment()->findById( (int) $cartItem[ static::WOO_ORDER_ITEM_META_KEY_MPA_PAYMENT_ID ] );

						if ( ! $mpaPayment || PaymentStatuses::STATUS_ABANDONED === $this->getPaymentStatus( $mpaPayment ) ) {

							WC()->cart->remove_cart_item( $cartItemKey );
							continue;
						}

						$mpaBooking = $mpaPayment->getBooking();

						if ( ! $mpaBooking || $mpaBooking->isConfirmed() ) {

							WC()->cart->remove_cart_item( $cartItemKey );
							continue;
						}
					}
				}
			},
			10
		);

		// Prevent add product to the cart without attached payment id
		add_action(
			'woocommerce_add_to_cart',
			function( $cartItemKey, $wooProductId, $quantity, $variationId, $variation, $cartItemData ) {

				if ( MPAWooCommerce::isWooCommerceProductForAppointmentPayments( $wooProductId ) ) {

					if ( ! array_key_exists( static::WOO_ORDER_ITEM_META_KEY_MPA_PAYMENT_ID, $cartItemData ) ) {
						throw new \Exception( __( 'Could not add an appointment to the cart without payment information.', 'mpa-woocommerce' ) );
					}

					$mpaPayment = mpapp()->repositories()->payment()->findById( (int) $cartItemData[ static::WOO_ORDER_ITEM_META_KEY_MPA_PAYMENT_ID ] );

					if ( ! $mpaPayment || PaymentStatuses::STATUS_ABANDONED === $this->getPaymentStatus( $mpaPayment ) ) {
						throw new \Exception( __( 'Could not add an appointment to the cart without payment information.', 'mpa-woocommerce' ) );
					}
				}
			},
			10,
			6
		);

		// Fill mpa_payment_id for woocommerce order item (cart item data -> legacy value -> order item meta data)
		add_action(
			'woocommerce_new_order_item',
			function( string $itemId, \WC_Order_Item $orderItem, int $orderId ) {

				if ( isset( $orderItem->legacy_values[ static::WOO_ORDER_ITEM_META_KEY_MPA_PAYMENT_ID ] ) ) {

					wc_update_order_item_meta(
						$itemId,
						static::WOO_ORDER_ITEM_META_KEY_MPA_PAYMENT_ID,
						$orderItem->legacy_values[ static::WOO_ORDER_ITEM_META_KEY_MPA_PAYMENT_ID ]
					);
				}
			},
			10,
			3
		);
	}

	private function customizeWooCommercePages() {
		// Force product invisibility to hide link to its page on the checkout and thank you pages
		add_filter(
			'woocommerce_product_is_visible',
			/**
			 * @param boolean $isProductVisible
			 * @param int $wooProductId
			 */
			function( $isProductVisible, $wooProductId ): bool {
				return MPAWooCommerce::isWooCommerceProductForAppointmentPayments( $wooProductId ) ? false : $isProductVisible;
			},
			10,
			2
		);

		// Make product in the order virtual to hide shipping info
		add_filter(
			'woocommerce_is_virtual',
			/**
			 * @param boolean $isProductVirtual
			 * @param \WC_Product $wooProduct
			 */
			function( $isProductVirtual, $wooProduct ): bool {
				return MPAWooCommerce::isWooCommerceProductForAppointmentPayments( $wooProduct->get_id() ) ? true : $isProductVirtual;
			},
			10,
			2
		);

		// Hide product on the shop page
		add_action(
			'pre_get_posts',
			function( \WP_Query $wpQuery ) {

				if ( is_admin() ||
				'product' !== $wpQuery->get( 'post_type' ) ||
				0 >= MPAWooCommerce::getWooCommerceProductId() ) {
					return;
				}

				$postIn    = $wpQuery->get( 'post__in' );
				$postNotIn = (array) $wpQuery->get( 'post__not_in' );

				$productIds = MPAWooCommerce::getAllLocalizedWooCommerceProductIds();

				if ( $postIn ) {

					$postIn = array_diff( $postIn, $productIds );
					$postIn = empty( $postIn ) ? array( 0 ) : $postIn;
					$wpQuery->set( 'post__in', $postIn );

				}

				if ( $postNotIn ) {

					$postNotIn = array_merge( $postNotIn, $productIds );
					$wpQuery->set( 'post__not_in', $postNotIn );
				}

				return $wpQuery;
			},
			90,
			1
		);

		// Prevent customer to change product quantity in the cart
		add_filter(
			'woocommerce_quantity_input_args',
			/**
			 * Change attr for WC quantity input
			 * @param array $args
			 * @param \WC_Product $wooProduct
			 * @return mixed
			 */
			function( $args, $wooProduct ) {

				if ( MPAWooCommerce::isWooCommerceProductForAppointmentPayments( $wooProduct->get_id() ) ) {
					$args['max_value'] = $args['input_value'];
					$args['min_value'] = $args['input_value'];
				}

				return $args;
			},
			10,
			2
		);

		// Hide product quantity on checkout page
		add_filter(
			'woocommerce_checkout_cart_item_quantity',
			/**
			* @param string $quantity
			* @param array $cartItemData
			* @param string $cartItemKey
			*/
			function( $quantity, $cartItemData, $cartItemKey ): bool {

				if ( MPAWooCommerce::isWooCommerceProductForAppointmentPayments( $cartItemData['product_id'] ) ) {
					$quantity = '';
				}
				return $quantity;
			},
			10,
			3
		);

		add_filter(
			'woocommerce_product_get_name',
			function ( string $wooProductName, \WC_Product $wooProduct ): string {

				if ( ! MPAWooCommerce::isWooCommerceProductForAppointmentPayments( $wooProduct->get_id() ) ) {
					return $wooProductName;
				}

				if ( null === WC()->cart ) {
					return $wooProductName;
				}

				$cartItems = WC()->cart->get_cart();

				foreach ( $cartItems as $cartItem ) {

					if ( $wooProduct->get_id() != $cartItem['product_id'] ) {
						continue;
					}

					$mpaPaymentId = isset( $cartItem[ static::WOO_ORDER_ITEM_META_KEY_MPA_PAYMENT_ID ] ) ? (int) $cartItem[ static::WOO_ORDER_ITEM_META_KEY_MPA_PAYMENT_ID ] : 0;
					$mpaPayment   = mpapp()->repositories()->payment()->findById( $mpaPaymentId );

					if ( null === $mpaPayment ) {
						continue;
					}

					$mpaBooking = $mpaPayment->getBooking();

					if ( null === $mpaBooking ) {
						continue;
					}

					$mpaReservations = $mpaBooking->getReservations();

					if ( 0 === count( $mpaReservations ) ) {
						continue;
					}

					$mpaFirstReservation = array_shift( $mpaReservations );

					$mpaCouponCode = $cartItem[ static::WOO_ORDER_ITEM_META_KEY_MPA_COUPON_CODE ] ?? '';
					$mpaDeposit    = $cartItem[ static::WOO_ORDER_ITEM_META_KEY_MPA_DEPOSIT ] ?? false;

					$dateFormatFromWPSettings = get_option( 'date_format' ); // e.g. "F j, Y"
					$timeFormatFromWPSettings = get_option( 'time_format' ); // e.g. "H:i:s"
					$eventStart               = $this->getEventStart( $mpaFirstReservation );

					$serviceName = get_the_title( $this->getServiceId( $mpaFirstReservation ) );

					$payingNow = '';
					$leftToPay = '';
					if ( $mpaDeposit ) {
						$payingNow       = wc_price( $mpaPayment->getAmount(), $this->getPaymentCurrency( $mpaPayment ) );
						$leftToPayAmount = $mpaBooking->getTotalPrice() - $mpaPayment->getAmount();
						$leftToPay       = wc_price( $leftToPayAmount, $this->getPaymentCurrency( $mpaPayment ) );
						if ( 0 >= $leftToPayAmount ) {
							$mpaDeposit = false;
						}
					}

					$wooProductName = esc_html__( 'Booking', 'motopress-appointment' ) . ' #' . $this->getPaymentBookingId( $mpaPayment ) .
								': ' . $serviceName . ', ' . $eventStart->format( $timeFormatFromWPSettings ) .
								' ' . $eventStart->format( $dateFormatFromWPSettings ) .
								( 1 < count( $mpaBooking->getReservations() ) ? '...' : '' ) .
								( $mpaCouponCode ? '<br>' . esc_html__( 'Coupon', 'motopress-appointment' ) . ': ' . $mpaCouponCode : '' ) .
								( $mpaDeposit ? '<br>' . esc_html__( 'Deposit', 'motopress-appointment' ) .
												'<br><b>' . esc_html__( 'Paying now', 'motopress-appointment' ) . ': ' . $payingNow . '</b>' .
												'<br><b>' . esc_html__( 'Left to pay', 'motopress-appointment' ) . ': ' . $leftToPay . '</b>' : '' );
				}

					return $wooProductName;
			},
			10,
			2
		);

		// Replace price on cart and checkout pages
		add_action(
			'woocommerce_before_calculate_totals',
			function( /* \WC_Cart */ $cart = null ) {

				if ( ! $cart ) {
					$cart = wc()->cart;
				}

				foreach ( $cart->cart_contents as $cartItem ) {

					if ( isset( $cartItem[ static::WOO_ORDER_ITEM_META_KEY_MPA_PAYMENT_ID ] ) ) {

						$mpaPayment = mpapp()->repositories()->payment()->findById( $cartItem[ static::WOO_ORDER_ITEM_META_KEY_MPA_PAYMENT_ID ] );

						if ( null != $mpaPayment ) {

							/** @var \WC_Product $cartItem['data'] */
							$cartItem['data']->set_price( $mpaPayment->getAmount() );
							break;
						}
					}
				}
			},
			10,
			1
		);

		// Fill WooCommerce Checkout billing fields from booking customer data
		add_filter(
			'woocommerce_checkout_get_value',
			function( $value, $fieldName ) {

				if ( ! in_array( $fieldName, array( 'billing_first_name', 'billing_phone', 'billing_email' ) ) ) {
					return $value;
				}

				foreach ( WC()->cart->get_cart() as $cartItem ) {

					if ( isset( $cartItem[ static::WOO_ORDER_ITEM_META_KEY_MPA_PAYMENT_ID ] ) ) {

						$mpaPayment = mpapp()->repositories()->payment()->findById( $cartItem[ static::WOO_ORDER_ITEM_META_KEY_MPA_PAYMENT_ID ] );

						if ( ! $mpaPayment ) {
							break;
						}

						$booking = $mpaPayment->getBooking();

						if ( ! $booking ) {
							break;
						}

						switch ( $fieldName ) {

							case 'billing_first_name':
								$value = $this->getCustomerName( $booking );
								break;

							case 'billing_phone':
								$value = $this->getCustomerPhone( $booking );
								break;

							case 'billing_email':
								$value = $this->getCustomerEmail( $booking );
								break;
						}
					}
				}

				return $value;
			},
			10,
			2
		);

		// Hide quantity on the Checkout thank you page
		add_filter(
			'woocommerce_order_item_quantity_html',
			function( $quantityHtml, /* WC_Order_Item */ $wcOrderItem ) {
				$orderItemData = $wcOrderItem->get_data();

				if ( isset( $orderItemData['product_id'] ) &&
					MPAWooCommerce::isWooCommerceProductForAppointmentPayments( $orderItemData['product_id'] ) ) {

					$quantityHtml = '';
				}
				return $quantityHtml;
			},
			10,
			2
		);

		// Hide mpa_payment_id from WooCommerce order item meta in the admin area and on the frontend
		add_filter(
			'woocommerce_order_item_get_formatted_meta_data',
			function( $wcOrderFormattedMetas ) {

				foreach ( $wcOrderFormattedMetas as $metaId => $metaData ) {

					if ( static::WOO_ORDER_ITEM_META_KEY_MPA_PAYMENT_ID == $metaData->key ) {

						unset( $wcOrderFormattedMetas[ $metaId ] );
						break;
					}
				}
				return $wcOrderFormattedMetas;
			},
			10,
			1
		);

		// Show link to the Appointment payment in the admin order details
		add_action(
			'woocommerce_after_order_itemmeta',
			function( $wcOrderItemId ) {

				$mpaPaymentId = wc_get_order_item_meta( $wcOrderItemId, static::WOO_ORDER_ITEM_META_KEY_MPA_PAYMENT_ID );

				if ( $mpaPaymentId ) {

					$mpaPaymentLink = sprintf( '<a href="%s">%s</a>', get_edit_post_link( $mpaPaymentId ), $mpaPaymentId );

					printf(
						// translators: %s is payment link html code
						'<p><strong>' . esc_html__( 'Appointment Booking Payment: %s', 'mpa-woocommerce' ) . '</strong></p>',
						wp_kses_post( $mpaPaymentLink )
					);
				}
			},
			10,
			1
		);
	}

	private function customizeWooCommercePaymentProcessing() {
		/**
		 * Fail payment
		 */
		add_action(
			'woocommerce_order_status_cancelled',
			function( $wcOrderId ) {
				$this->processPaymentForWooCommerceOrderId( $wcOrderId );
			},
			10,
			1
		);

		add_action(
			'woocommerce_order_status_failed',
			function( $wcOrderId ) {
				$this->processPaymentForWooCommerceOrderId( $wcOrderId );
			},
			10,
			1
		);

		add_action(
			'trashed_post',
			function( $wcOrderId ) {
				if ( 'shop_order' === get_post_type( $wcOrderId ) ) {

					$this->processPaymentForWooCommerceOrderId( $wcOrderId );
				}
			},
			10,
			1
		);

		add_action(
			'woocommerce_delete_order_items',
			function( $wcOrderId ) {
				if ( 'shop_order' === get_post_type( $wcOrderId ) ) {

					$this->processPaymentForWooCommerceOrderId( $wcOrderId );
				}
			},
			10,
			1
		);

		/**
		 * Waiting for payment
		 */
		add_action(
			'woocommerce_order_status_on-hold',
			function( $wcOrderId ) {
				$this->processPaymentForWooCommerceOrderId( $wcOrderId );
			},
			10,
			1
		);

		add_action(
			'woocommerce_order_status_pending',
			function( $wcOrderId ) {
				$this->processPaymentForWooCommerceOrderId( $wcOrderId );
			},
			10,
			1
		);

		add_action(
			'woocommerce_order_status_processing',
			function( $wcOrderId ) {
				$this->processPaymentForWooCommerceOrderId( $wcOrderId );
			},
			10,
			1
		);

		add_action(
			'woocommerce_checkout_order_processed',
			function( $wcOrderId ) {
				$this->processPaymentForWooCommerceOrderId( $wcOrderId );
			},
			10,
			1
		);

		/**
		 * Complete payment
		 */
		add_action(
			'woocommerce_order_status_completed',
			function( $wcOrderId ) {
				$this->processPaymentForWooCommerceOrderId( $wcOrderId );
			},
			10,
			1
		);

		/**
		 * Refund payment
		 */
		add_action(
			'woocommerce_order_status_refunded',
			function( $wcOrderId ) {
				$this->processPaymentForWooCommerceOrderId( $wcOrderId );
			},
			10,
			1
		);
	}

	private function processPaymentForWooCommerceOrderId( $wcOrderId ) {

		$wcOrder = wc_get_order( $wcOrderId );

		$mpaPayment = $this->getPaymentFromWooCommerceOrder( $wcOrder );

		if ( null === $mpaPayment ) {
			return;
		}

		$this->processPayment(
			$mpaPayment,
			array(
				'booking'  => $mpaPayment->getBooking(),
				'wc_order' => $wcOrder,
			)
		);
	}

	private function getPaymentFromWooCommerceOrder( $wcOrder ): ?Payment {

		if ( empty( $wcOrder ) ) {
			return null;
		}

		$wcOrderItems = $wcOrder->get_items();

		foreach ( $wcOrderItems as $wcOrderItem ) {

			$orderItemData = $wcOrderItem->get_data();

			if ( isset( $orderItemData['product_id'] ) &&
				MPAWooCommerce::isWooCommerceProductForAppointmentPayments( $orderItemData['product_id'] ) ) {

				$mpaPaymentId = wc_get_order_item_meta( $wcOrderItem->get_id(), static::WOO_ORDER_ITEM_META_KEY_MPA_PAYMENT_ID );

				return mpapp()->repositories()->payment()->findById( $mpaPaymentId );
			}
		}
		return null;
	}


	public function printBillingFields() {}


	public function isActive() {

		return $this->isEnabled() &&
			0 < MPAWooCommerce::getWooCommerceProductId() &&
			get_woocommerce_currency() === mpapp()->settings()->getCurrency();
	}

	/**
	 * Each gateway can add here additional payment data for a new starting payment.
	 * @throws \Exception if payment had not been stored
	 */
	protected function prepareAndStoreNewPayment( Payment $payment, Booking $booking, array $paymentData ): Payment {

		// return the status to auto-draft, because the payment will be prepared by an external plugin.
		$payment->setStatus( 'auto-draft' );

		return parent::prepareAndStoreNewPayment( $payment, $booking, $paymentData );
	}

	/**
	 * @param Payment $payment
	 * @param array $paymentDetails where $paymentDetails['booking'] contains \MotoPress\Appointment\Entities\Booking
	 *              $paymentDetails['wc_order'] contains WC_Order or WC_Order_Refund
	 * @return bool|WP_Error
	 */
	public function processPayment( $payment, $paymentDetails ) {

		if ( empty( $paymentDetails['wc_order'] ) ) {
			// phpcs:ignore
			error_log( __METHOD__ . ': Something went wrong and $paymentDetails[\'wc_order\'] is empty!' );
			return false;
		}

		$wcOrder = $paymentDetails['wc_order'];
		$booking = $paymentDetails['booking'];

		// Update booking and payment to pending status and fill some data from WooCommerce Order
		// Do it instead of BookingRestController::createBooking()
		if ( 'auto-draft' === $this->getPaymentStatus( $payment ) ) {

			$name = $wcOrder->get_billing_first_name() . ' ' . $wcOrder->get_billing_last_name();
			$this->setCustomerName( $booking, $name );

			$email = $wcOrder->get_billing_email();
			try {
				$this->setCustomerEmail( $booking, $email );
			// phpcs:ignore
			} catch ( \Exception $e ) {
			}

			$phone = $wcOrder->get_billing_phone();
			$this->setCustomerPhone( $booking, $phone );

			$status = mpapp()->settings()->getDefaultBookingStatus();
			$this->setStatus( $booking, $status );

			$bookingService = new BookingService();
			$bookingService->saveBooking( $booking );

			$transactionId = $wcOrder->get_id();
			$payment->setTransactionId( $transactionId );

			$paymentMethod = $wcOrder->get_payment_method();
			$payment->setPaymentMethod( $paymentMethod );

			$payment = $this->storePayment( $payment );
			$booking->expectPayment( $payment );
		}

		switch ( $wcOrder->get_status() ) {
			case 'on-hold':
			case 'pending':
				mpa_payment_manager()->holdPayment(
					$payment,
					// translators: %d is woocommerce order id
					sprintf( esc_html__( 'Waiting for WooCommerce order [id = %d] confirmation.', 'mpa-woocommerce' ), $wcOrder->get_id() ),
					true
				);
				break;

			case 'processing':
				mpa_payment_manager()->holdPayment(
					$payment,
					// translators: %d is woocommerce order id
					sprintf( esc_html__( 'Waiting for WooCommerce order [id = %d] completion.', 'mpa-woocommerce' ), $wcOrder->get_id() ),
					true
				);

				// if booking paid by cach on delivery then we complete booking
				if ( 'cod' === $this->getPaymentMethod( $payment ) ) {

					mpa_update_status( $booking, BookingStatuses::STATUS_CONFIRMED );

					/**
					 * @param Booking $booking
					 * @param Payment $payment
					 */
					do_action( 'mpa_booking_confirmed_with_payment', $booking, $payment );
				}
				break;

			case 'completed':
				mpa_payment_manager()->completePayment(
					$payment,
					// translators: %d is woocommerce order id
					sprintf( esc_html__( 'WooCommerce order [id = %d] is completed.', 'mpa-woocommerce' ), $wcOrder->get_id() ),
					true
				);
				break;

			case 'cancelled':
			case 'trash':
				mpa_payment_manager()->cancelPayment(
					$payment,
					// translators: %d is woocommerce order id
					sprintf( esc_html__( 'WooCommerce order [id = %d] is canceled.', 'mpa-woocommerce' ), $wcOrder->get_id() ),
					true
				);
				break;

			case 'failed':
				mpa_payment_manager()->failPayment(
					$payment,
					// translators: %d is woocommerce order id
					sprintf( esc_html__( 'WooCommerce order [id = %d] is failed.', 'mpa-woocommerce' ), $wcOrder->get_id() ),
					true
				);
				break;

			case 'refunded':
				mpa_payment_manager()->refundPayment(
					$payment,
					// translators: %d is woocommerce order id
					sprintf( esc_html__( 'WooCommerce order [id = %d] is refunded.', 'mpa-woocommerce' ), $wcOrder->get_id() ),
					true
				);
				break;
		}

		return true;
	}
}
