<?php
/**
 * Plugin Name: Appointment Booking WooCommerce Payments
 * Plugin URI: https://motopress.com/products/appointment-booking-woocommerce-payments/
 * Description: Connect WooCommerce payment gateways to process appointment bookings.
 * Version: 1.1.2
 * Requires at least: 5.7
 * Requires PHP: 7.1
 * Author: MotoPress
 * Author URI: https://motopress.com/
 * License: GPLv2 or later
 * Text Domain: mpa-woocommerce
 * Domain Path: /languages
 */

namespace MotoPress\Appointment\Addons\WooCommerce;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

final class MPAWooCommerce {

	const SETTINGS_NAME_PLUGIN_LICENSE_KEY = 'mpa_woocommerce_payment_gateway_license_key';

	private static $pluginClasses = array(
		'BackendHelper'              => 'admin/backend-helper.php',
		'PluginLifecycleHandler'     => 'admin/plugin-lifecycle-handler.php',
		'EDDPluginUpdater'           => 'admin/edd-plugin-updater.php',
		'PluginLicenseSettingsField' => 'admin/plugin-license-settings-field.php',
		'WooCommercePaymentsGateway' => 'includes/woocommerce-payments-gateway.php',
	);

	private static $instance;

	private $pluginDir;
	private $pluginUrl;

	// prevent cloning of singleton
	public function __clone() {}
	public function __wakeup() {}

	public static function getInstance() {

		if ( ! isset( self::$instance ) ) {

			self::$instance = new MPAWooCommerce();
		}

		return self::$instance;
	}

	private function __construct() {

		$this->pluginDir = plugin_dir_path( __FILE__ );
		$this->pluginUrl = plugin_dir_url( __FILE__ );

		spl_autoload_register(
			function( $class ) {
				$this->loadPluginClass( $class );
			},
			true,
			true
		);

		add_action(
			'plugins_loaded',
			function() {
				load_plugin_textdomain(
					'mpa-woocommerce',
					false,
					$this->pluginDir . 'languages/'
				);

				$pluginLifecycleHandler = new PluginLifecycleHandler( __FILE__ );

				// do not initialize plugin if environment does not suite it
				if ( ! $pluginLifecycleHandler->isWPEnvironmentSuitedForPlugin() ) {
					return;
				}

				// register payment gateway
				add_filter(
					'mpa_registered_payment_gateways',
					function( $paymentGateways ) {
						$paymentGateway                              = new WooCommercePaymentsGateway();
						$paymentGateways[ $paymentGateway->getId() ] = $paymentGateway;

						return $paymentGateways;
					}
				);

			},
			1
		);
	}

	/**
	 * @return false or string with class file path
	 */
	private function loadPluginClass( string $classNameWithNamespace ) {

		$classNameWithNamespace = ltrim( $classNameWithNamespace, '\\' );

		if ( false === strpos( $classNameWithNamespace, __NAMESPACE__ ) ) {
			return false;
		}

		$classNameWithoutNamespace = str_replace( __NAMESPACE__ . '\\', '', $classNameWithNamespace );

		if ( ! empty( static::$pluginClasses[ $classNameWithoutNamespace ] ) ) {

			$classFilePath = $this->pluginDir . static::$pluginClasses[ $classNameWithoutNamespace ];

			if ( file_exists( $classFilePath ) ) {

				require_once $classFilePath;

				return $classFilePath;
			}
		}

		return false;
	}

	public static function getPluginDir(): string {
		return static::getInstance()->pluginDir;
	}

	public static function getPluginUrl(): string {
		return static::getInstance()->pluginUrl;
	}

	public static function getProductId(): string {
		return '1318378';
	}

	public static function getPluginAuthorName(): string {
		$pluginData = get_plugin_data( __FILE__, false, false );
		return isset( $pluginData['Author'] ) ? $pluginData['Author'] : '';
	}

	public static function getPluginVersion(): string {
		$pluginData = get_plugin_data( __FILE__, false, false );
		return isset( $pluginData['Version'] ) ? $pluginData['Version'] : '';
	}

	public static function getPluginSourceServerUrl(): string {
		$pluginData = get_plugin_data( __FILE__, false, false );
		return isset( $pluginData['PluginURI'] ) ? $pluginData['PluginURI'] : '';
	}

	public static function getWooCommerceProductId(): int {
		return (int) get_option( 'mpa_woocommerce_payment_gateway_woo_product_id', 0 );
	}

	public static function getLocalizedWooCommerceProductId(): int {

		$localizedWooCommerceProductId = static::getWooCommerceProductId();

		$currentLanguage = apply_filters( 'wpml_current_language', null );

		if ( ! is_null( $currentLanguage ) ) {

			$localizedWooCommerceProductId = (int) apply_filters(
				'wpml_object_id',
				$localizedWooCommerceProductId,
				'product',
				true,
				$currentLanguage
			);
		}

		return $localizedWooCommerceProductId;
	}

	/**
	 * @return array [%Language code% => %Product ID%]
	 */
	public static function getAllLocalizedWooCommerceProductIds(): array {

		$appointmentWooProductId = static::getWooCommerceProductId();

		if ( 0 >= $appointmentWooProductId ) {
			return array();
		}

		$wooProductIds = array();

		$languages     = apply_filters( 'wpml_active_languages', array() );
		$languageCodes = wp_list_pluck( $languages, 'language_code' );

		foreach ( $languageCodes as $languageCode ) {

			$localizedAppointmentWooProductId = (int) apply_filters( 'wpml_object_id', $appointmentWooProductId, 'product', true, $languageCode );

			if ( 0 < $localizedAppointmentWooProductId ) {

				$wooProductIds[ $languageCode ] = $localizedAppointmentWooProductId;
			}
		}

		return $wooProductIds;
	}

	public static function isWooCommerceProductForAppointmentPayments( int $wooCommerceProductId ): bool {

		$appointmentWooProductId = static::getWooCommerceProductId();

		if ( 0 >= $appointmentWooProductId ) {
			return false;
		}

		if ( $appointmentWooProductId == $wooCommerceProductId ) {
			return true;
		}

		$languages     = apply_filters( 'wpml_active_languages', array() );
		$languageCodes = wp_list_pluck( $languages, 'language_code' );

		foreach ( $languageCodes as $languageCode ) {

			$localizedAppointmentWooProductId = (int) apply_filters( 'wpml_object_id', $appointmentWooProductId, 'product', true, $languageCode );

			if ( $localizedAppointmentWooProductId == $wooCommerceProductId ) {
				return true;
			}
		}

		return false;
	}

	public static function isEDDLicenseEnabled(): bool {
		return (bool) apply_filters( 'mpawoo_use_edd_license', true );
	}

	public static function getLicenseKey(): string {
		return get_option( static::SETTINGS_NAME_PLUGIN_LICENSE_KEY, '' );
	}
}

// init plugin
MPAWooCommerce::getInstance();
